import sys, os, glob, os.path
import distutils.util
import doctest

build_dir = "build/lib.%s-%s" % ( distutils.util.get_platform(), sys.version[0:3] )

sys.path.insert( 0, os.path.join( os.getcwd(), build_dir ) )
import HTSeq
os.chdir( "example_data" )

def test_rst_file( filename ):
   print "Doctest of %s:" % os.path.basename( filename )
   (failure_count, test_count) = doctest.testfile( 
      os.path.join( "..", "doc", filename ), module_relative=False )

   if failure_count == 0:
      print "All %d tests passed." % test_count
      return True
   else:   
      print "%d of %d tests failed." % (failure_count, test_count)
      return False

ok = True
if len(sys.argv) == 1:
   for fn in glob.glob( "../doc/*.rst" ):
      ok &= test_rst_file( os.path.basename( fn ) )
      print
   if not ok:
      print "Not all tests passed."
      exit( 1 )
elif len(sys.argv) == 2:
   test_rst_file( sys.argv[1] )
else:
   print "Wrong usage"
   print "Call without arguments to run all doctest, or with the (base) name"
   print "of one rst file from the doc directory to run doctest on it."



import sys
import os
import math
import re
import csv
import gzip
import itertools 
import collections
import cStringIO
import warnings
import numpy
cimport numpy

import StepVector
import _HTSeq_internal


###########################
##   GenomicInterval 
###########################

cdef str strand_plus = intern( "+" )
cdef str strand_minus = intern( "-" )
cdef str strand_nostrand = intern( "." )

cdef class GenomicInterval:

   """A GenomicInterval specifies an interval (i.e., a range of 
   consecutive positions) on a reference genome.
   
   A GenomicInterval object has the following slots, some of which 
   are calculated from the other:
     
      chrom: The name of a sequence (i.e., chromosome, contig, or 
         the like). 
      start: The start of the interval. Even on the reverse strand,
         this is always the smaller of the two values 'start' and 'end'.
         Note that all positions should be given as 0-based value!
      end: The end of the interval. Following Python convention for 
         ranges, this in one more than the coordinate of the last base
         that is considered part of the sequence.
      strand: The strand, as a single character, '+' or '-'. '.' indicates
         that the strand is irrelavant. (Alternatively, pass a Strand object.)
      length: The length of the interval, i.e., end - start
      start_d: The "directional start" position. This is the position of the
        first base of the interval, taking the strand into account. Hence, 
        this is the same as 'start' except when strand == '-', in which 
        case it is end-1.
      end_d: The "directional end": Usually, the same as 'end', but for 
        strand=='-1', it is start-1.
   """
   
   def __init__( GenomicInterval self, str chrom, long start, long end, 
         str strand = strand_nostrand ):
      """See the class docstring for the meaning of the slots. Note that 
      there is also a factory function, 'from_directional', to be used if
      you wish to specify start_d and length.
      """
      self.chrom = intern( chrom )
      self.start = start
      self.end = end
      self.strand = strand
      if self.start > self.end:
         raise ValueError, "start is larger than end"
            
   property strand:
      def __set__( self, strand ):
         strand = intern( strand )
         if not( strand is strand_plus or strand is strand_minus or 
               strand is strand_nostrand ):
            raise ValueError, "Strand must be'+', '-', or '.'."
         self._strand = strand
      def __get__( self ):
         return self._strand

   def __reduce__( GenomicInterval self ):
      return GenomicInterval, ( self.chrom, self.start, self.end, 
         self.strand )
         
   def __copy__( self ):
      constr, args = self.__reduce__()
      return constr( *args )   
   
   def __repr__( GenomicInterval self ):
      return "<%s object '%s', [%d,%s), strand '%s'>" % \
         ( self.__class__.__name__, self.chrom, self.start, 
           str(self.end) if self.end != sys.maxint else "Inf", self.strand )
         
   def __str__( GenomicInterval self ):
         return "%s:[%d,%s)/%s" % \
            ( self.chrom, self.start, str(self.end) if self.end != sys.maxint else "Inf", self.strand )

   property length:

      """The length is calculated as end - start. If you set the length, 
      'start_d' will be preserved, i.e., 'end' is changed, unless the strand
      is '-', in which case 'start' is changed."""   
            
      def __get__( GenomicInterval self ):
         return self.end - self.start

      def __set__( GenomicInterval self, long newLength ):
         if self._strand is not strand_minus:
            self.end = self.start + newLength
         else:
            self.start = self.end - newLength

   property start_d:
      """See the class docstring for the meaning of the 'directional start'.
      Note that if you set 'start_d', both the start and the end are changed, 
      such the interval gets the requested new directional start and its
      length stays unchanged."""

      def __get__( GenomicInterval self ):
         if self._strand is not strand_minus:
            return self.start
         else:
            return self.end - 1

      def __set__( GenomicInterval self, long newStartd ):
         if self._strand is not strand_minus:
            self.end = newStartd + self.length
            self.start = newStartd
         else:
            self.start = newStartd + 1 - self.length
            self.end = newStartd + 1
         
   property end_d:
   
      def __get__( GenomicInterval self ):
         if self._strand is not strand_minus:
            return self.end
         else:
            return self.start - 1
            
   property start_as_pos:
      def __get__( GenomicInterval self ):
         return GenomicPosition( self.chrom, self.start, self. strand )            

   property end_as_pos:
      def __get__( GenomicInterval self ):
         return GenomicPosition( self.chrom, self.end, self. strand )            

   property start_d_as_pos:
      def __get__( GenomicInterval self ):
         return GenomicPosition( self.chrom, self.start_d, self. strand )            

   property end_d_as_pos:
      def __get__( GenomicInterval self ):
         return GenomicPosition( self.chrom, self.end_d, self. strand )            

         
   def __richcmp__( GenomicInterval self, GenomicInterval other, int op ):
      if op == 2:  # ==
         if other == None:
            return False
         return self._strand is other._strand and \
            self.start == other.start and self.end == other.end
      elif op == 3:  # !=
         return not ( self == other )
      else:
         raise NotImplementedError
         
   def __hash__( GenomicInterval self ):
      return hash( ( self.chrom, self.start, self.end, self.strand ) )
           
   cpdef is_contained_in( GenomicInterval self, GenomicInterval iv ):
      """Returns a boolean value indicating whether the 'self' interval 
      is fully within the 'iv' interval.
      
      This is deemed the case if
        - both are on the same chromosome, and    
        - both are on the same strand, or at least one of them is
           not stranded (i.e., has strand == '.'), and
        - self.start >= iv.start, and
        - self.end <= iv.end
      """
      if iv == None:
         return False
      if self.chrom != iv.chrom:
         return False
      if self._strand is not strand_nostrand and iv.strand is not strand_nostrand and \
            self.strand is not iv._strand:
         return False 
      if self.start < iv.start or self.end > iv.end:
         return False
      return True

   cpdef contains( GenomicInterval self, GenomicInterval iv ):
      """Returns a boolean value indicating whether the 'self' interval 
      fully contains the 'iv' interval.

      See 'is_contained_in' for the exact criteria.
      """
      if iv == None:
            return False
      return iv.is_contained_in( self )

   cpdef overlaps( GenomicInterval self, GenomicInterval iv ):
      """Returns a boolean value indicating whether the 'self' interval 
      overlaps the 'iv' interval.
      
      This is deemed the case if
        - both are on the same chromosome, and    
        - both are on the same strand, or at least one of them is
           not stranded (i.e., has strand == '.'), and
        - the actual intervals overlap
      """
      if iv == None:
         return False
      if self.chrom != iv.chrom:
         return False
      if self.strand is not strand_nostrand and iv.strand is not strand_nostrand and \
            self.strand is not iv.strand:
         return False 
      if self.start <= iv.start:
         return self.end > iv.start
      else: 
         return iv.end > self.start
      
   def xrange( GenomicInterval self, long int step = 1 ):
      """Generate an iterator over the GenomicPositions covered by the interval,
      running from start to end.
      """
      return _HTSeq_internal.GenomicInterval_xrange( self, step )

   def xrange_d( GenomicInterval self, long int step = 1 ):
      """Generate an iterator over the GenomicPositions covered by the interval.
      running from start_d to end_d.
      """
      return _HTSeq_internal.GenomicInterval_xranged( self, step )
      
   cpdef extend_to_include( GenomicInterval self, GenomicInterval iv ):
      """Extend the interval such that it includes iv."""
      if iv is None:
         raise TypeError, "Cannot extend an interval to include None."
      if self.chrom != iv.chrom:
         raise ValueError, "Cannot extend an interval to include an interval on another chromosome."
      if self.strand is not strand_nostrand and iv.strand is not strand_nostrand and \
            self.strand is not iv.strand:
         raise ValueError, "Cannot extend an interval to include an interval on another strand."
      self.start = min( self.start, iv.start )
      self.end = max( self.end, iv.end )
      
   def copy( self ):
      return GenomicInterval( self.chrom, self.start, self.end, self.strand )


def GenomicInterval_from_directional( str chrom, long int start_d, long int length, str strand="." ):
   strand = intern( strand )
   if strand.se is not strand_minus:
      return GenomicInterval( chrom, start_d, start_d+length, strand )
   else:
      return GenomicInterval( chrom, start_d-length+1, start_d+1, strand )


cdef class GenomicPosition( GenomicInterval ):

   """A GenomicPosition specifies the position of a nucleotide or
   base pair on a reference genome.
   
   It has the following slots:
      chrom: The name of a sequence (i.e., chromosome, contig, or 
         the like). 
      pos: The position on the sequence specified by seqname.
         The position should always be given as 0-based value!
      strand: The strand, as a single character, '+' or '-'. '.' indicates
         that the strand is irrelavant.

   The GenomicPosition class is derived from GenomicInterval. Hence,
   a GenomicPosition is always a GenomicInterval of length 1. Do not tinker
   with the exposed GenomeInterval slots.
   """

   def __init__( self, str chrom, long int pos, str strand='.' ):
      GenomicInterval.__init__( self, chrom, pos, pos+1, strand )
      
   property pos:
   
      """As GenomicPosition is a subclass of GenomicInterval, 'pos' is actually
      just an alias for 'start_d'.
      """
   
      def __get__( self ):
         return self.start_d
         
      def __set__( self, long newValue ):
         self.start_d = newValue
      
   property end:
   
      def __get__( self ):
         return self.start + 1

   property length:
   
      def __get__( self ):
         return 1
      
   def __repr__( self ):
      return "<%s object '%s':%d, strand '%s'>" % \
         ( self.__class__.__name__, self.chrom, self.pos, self.strand )               
            
   def __str__( self ):
      return "%s:%d/%s" % ( self.chrom, self.pos, self.strand )            

   def __reduce__( GenomicPosition self ):
      return GenomicPosition, ( self.chrom, self.pos, self.strand )
   
   def copy( self ):
      return GenomicPosition( self.chrom, self.pos, self.strand )
      

cdef class ChromVector( object ):

   cdef public object array 
   cdef public GenomicInterval iv
   cdef public int offset
   cdef public bint is_vector_of_sets
   cdef public str _storage

   @classmethod 
   def create( cls, GenomicInterval iv, str typecode, str storage, str memmap_dir = "" ):
      ncv = cls()
      ncv.iv = iv
      if storage == "ndarray":
         if typecode != 'O':
            ncv.array = numpy.zeros( shape = ( iv.length, ), dtype = typecode )
         else:
            ncv.array = numpy.empty( shape = ( iv.length, ), dtype = typecode )
            ncv.array[:] = None
      elif storage == "memmap":
         ncv.array = numpy.memmap( shape = ( iv.length, ), dtype = typecode, 
            filename = os.path.join( memmap_dir, iv.chrom + iv.strand + ".nmm" ), mode='w+' )
      elif storage == "step":
         ncv.array = StepVector.StepVector.create( typecode = typecode )
      else:
         raise ValueError, "Illegal storage mode."
      ncv._storage = storage
      # TODO: Test whether offset works properly
      ncv.offset = iv.start
      ncv.is_vector_of_sets = False
      return ncv
   
   @classmethod 
   def _create_view( cls, ChromVector vec, GenomicInterval iv ):
      if iv.length == 0:
         raise IndexError, "Cannot subset to zero-length interval."
      v = cls()
      v.iv = iv
      v.array = vec.array
      v.offset = vec.offset
      v.is_vector_of_sets = vec.is_vector_of_sets
      v._storage = vec._storage      
      return v

   def __getitem__( self, index ):
      cdef slice index_slice
      cdef long int index_int
      cdef long int start, stop
      cdef GenomicInterval iv
      if isinstance( index, int ):
         index_int = index
         if index_int < self.iv.start or index_int >= self.iv.end:
            raise IndexError
         return self.array[ index_int - self.offset ]
      elif isinstance( index, slice ):
         index_slice = index
         if index_slice.start is not None:
            start = index_slice.start
            if start < self.iv.start:
               raise IndexError, "start too small"
         else:
            start = self.iv.start
         if index_slice.stop is not None:
            stop = index_slice.stop
            if stop > self.iv.end:
               raise IndexError, "stop too large"               
         else:
            stop = self.iv.end
         iv = GenomicInterval( self.iv.chrom, start, stop, self.iv.strand )
         if not self.iv.contains( iv ):
            raise IndexError
         return ChromVector._create_view( self, iv )
      elif isinstance( index, GenomicInterval ):
         if not self.iv.contains( index ):
            raise IndexError
         if self.iv.strand is strand_nostrand and \
               index.strand is not strand_nostrand:
            iv = index.copy()   # Is this correct now?
            iv.strand = strand_nostrand
         return ChromVector._create_view( self, iv )
      else:
         raise TypeError, "Illegal index type"
   
   def __setitem__( self, index, value ):
      cdef slice index_slice
      cdef long int start, stop
      if isinstance( value, ChromVector ):       
         if self.array is value.array and value.iv.start == index.start and \
               value.iv.end == index.stop and ( index.step is None or index.step == 1 ):
            return
         else:
            raise NotImplementedError, "Required assignment signature not yet implemented."
      if isinstance( index, int ):
         self.array[ index - self.iv.start ] = value
      elif isinstance( index, slice ):
         index_slice = index
         if index_slice.start is not None:
            start = index_slice.start
            if start < self.iv.start:
               raise IndexError, "start too small"
         else:
            start = self.iv.start
         if index_slice.stop is not None:
            stop = index_slice.stop
            if stop > self.iv.end:
               raise IndexError, "stop too large"
         else:
            stop = self.iv.end
         if start > stop:
            raise IndexError, "Start of interval is after its end."
         if start == stop:
            raise IndexError, "Cannot assign to zero-length interval."
         self.array[ start - self.offset : stop - self.iv.start : index.step ] = value
      elif isinstance( index, GenomicInterval ):
         if index.chrom != self.iv.chrom:
            raise KeyError, "Chromosome name mismatch."
         if self.iv.strand is not strand_nostrand and \
               self.iv.strand is not self.index.strand:
            raise KeyError, "Strand mismatch."
         self.array[ index.iv.start - self.iv.start, 
            index.iv.end - self.iv.start ] = value
      else:
         raise TypeError, "Illegal index type"

   def __iadd__( self, value ):
      if not self.is_vector_of_sets:
         self.array[ self.iv.start - self.offset : self.iv.end - self.offset ].__iadd__( value )
      else:
         def addval( x ):
            y = x.copy()
            y.add( value )
            return y
         self.apply( addval )
      return self
      
   def __iter__( self ):
      return self.values()
      
   def values( self ):
      return iter( self.array[ self.iv.start - self.offset : self.iv.end - self.offset ] )
   
   def steps( self ):
      return _HTSeq_internal.ChromVector_steps( self )
      
   def apply( self, fun ):
      for iv, value in self.steps():
         self.array[ iv.start - self.offset : iv.end - self.offset ] = fun( value )
         
   def __repr__( self ):
      return "<%s object, %s, %s>" % ( self.__class__.__name__, str(self.iv), self._storage )
         
   def __reduce__( self ):
      assert self.__class__ is ChromVector
      return( _ChromVector_unpickle, 
         ( self.array, self.iv, self.offset, self.is_vector_of_sets, self._storage ) )
         
def _ChromVector_unpickle( array, iv, offset, is_vector_of_sets, _storage ):
   cv = ChromVector()
   cv.array =  array 
   cv.iv = iv
   cv.offset = offset
   cv.is_vector_of_sets = is_vector_of_sets
   cv._storage = _storage
   return cv
   
cdef class GenomicArray( object ):
   
   cdef public dict chrom_vectors
   cdef readonly bint stranded
   cdef readonly str typecode
   cdef public bint auto_add_chroms
   cdef readonly str storage
   cdef readonly str memmap_dir
   
   def __init__( self, object chroms, bint stranded=True, str typecode='d',
         str storage='step', str memmap_dir = "" ):
      self.chrom_vectors = {}
      self.stranded = stranded
      self.typecode = typecode
      self.auto_add_chroms = chroms == "auto"
      if self.auto_add_chroms:      
         chroms = []
         if storage != 'step':
            raise TypeError, "Automatic adding of chromosomes can " + \
               " only be used with storage type 'StepVector'."
      elif isinstance( chroms, list ):
         if storage != 'step':
            raise TypeError, "Indefinite-length chromosomes can " + \
               " only be used with storage type 'StepVector'."
         chroms = dict( [ ( c, sys.maxint ) for c in chroms ] )
      elif not isinstance( chroms, dict ):
         raise TypeError, "'chroms' must be a list or a dict or 'auto'."
      self.storage = storage
      self.memmap_dir = memmap_dir
      
      for chrom in chroms:
         self.add_chrom( chrom, chroms[chrom] )
            
   def __getitem__( self, index ):
      if isinstance( index, GenomicInterval ):
         if self.stranded and index.strand not in ( strand_plus, strand_minus ):
            raise KeyError, "Non-stranded index used for stranded GenomicArray."
         if self.auto_add_chroms and index.chrom not in self.chrom_vectors:
            self.add_chrom( index.chrom )
         if isinstance( index, GenomicPosition ):
            if self.stranded:
               return self.chrom_vectors[ index.chrom ][ index.strand ][ index.pos ]
            else:
               return self.chrom_vectors[ index.chrom ][ strand_nostrand ][ index.pos ]
         else:
            if self.stranded:
               return self.chrom_vectors[ index.chrom ][ index.strand ][ index.start : index.end ]
            else:
               return self.chrom_vectors[ index.chrom ][ strand_nostrand ][ index.start : index.end ]
      else:
         return self.chrom_vectors[ index ]

   def __setitem__( self, index, value ):
      cdef GenomicInterval index2
      if isinstance( value, ChromVector ): 
         if not isinstance( index, GenomicInterval ):
            raise NotImplementedError, "Required assignment signature not yet implemented."
         index2 = index.copy()
         if not self.stranded:
            index2.strand = strand_nostrand
         if self.chrom_vectors[ index2.chrom ][ index2.strand ].array is value.array and index2 == value.iv:
            return
         raise NotImplementedError, "Required assignment signature not yet implemented."
      if isinstance( index, GenomicInterval ):
         if self.stranded and index.strand not in ( strand_plus, strand_minus ):
            raise KeyError, "Non-stranded index used for stranded GenomicArray."
         if self.auto_add_chroms and index.chrom not in self.chrom_vectors:
            self.add_chrom( index.chrom )
         if self.stranded:
            self.chrom_vectors[ index.chrom ][ index.strand ][ index.start : index.end ] = value
         else:
            self.chrom_vectors[ index.chrom ][ strand_nostrand ][ index.start : index.end ] = value
      else:
         raise TypeError, "Illegal index type."
            
   def add_chrom( self, chrom, length = sys.maxint, start_index = 0 ):
      cdef GenomicInterval iv 
      if length == sys.maxint:
         iv = GenomicInterval( chrom, start_index, sys.maxint, "." )
      else:
         iv = GenomicInterval( chrom, start_index, start_index + length, "." )
      if self.stranded:
         self.chrom_vectors[ chrom ] = {}
         iv.strand = "+"
         self.chrom_vectors[ chrom ][ strand_plus ] = \
            ChromVector.create( iv, self.typecode, self.storage, self.memmap_dir )
         iv = iv.copy()
         iv.strand = "-"
         self.chrom_vectors[ chrom ][ strand_minus ] = \
            ChromVector.create( iv, self.typecode, self.storage, self.memmap_dir )
      else:   
         self.chrom_vectors[ chrom ] = {
            strand_nostrand:  ChromVector.create( iv, self.typecode, self.storage ) }
   
   def __reduce__( self ):
      return ( _GenomicArray_unpickle, ( self.stranded, self.typecode, self.chrom_vectors ) )
      
   def write_bedgraph_file( self, file_or_filename, strand=".", track_options="" ):
      if ( not self.stranded ) and strand != ".":
         raise ValueError, "Strand specified in unstranded GenomicArray."
      if self.stranded and strand not in ( strand_plus, strand_minus ):
         raise ValueError, "Strand must be specified for stranded GenomicArray."
      if hasattr( file_or_filename, "write" ):
         f = file_or_filename
      else:
         f = open( file_or_filename, "w" )
      if track_options == "":
         f.write( "track type=bedGraph\n" )
      else:
         f.write( "track type=bedGraph %s\n" % track_options )
      for chrom in self.chrom_vectors:
         for iv, value in self.chrom_vectors[ chrom ][ strand ].steps():
            if iv.start == -sys.maxint-1 or iv.end == sys.maxint:
               continue
            f.write( "%s\t%d\t%d\t%f\n" % (iv.chrom, iv.start, iv.end, value) )    
      if not hasattr( file_or_filename, "write" ):
         f.close()
         
   def steps( self ):
      return _HTSeq_internal.GenomicArray_steps( self )
      
       
def _GenomicArray_unpickle( stranded, typecode, chrom_vectors ):
   ga = GenomicArray( {}, stranded, typecode )
   ga.chrom_vectors = chrom_vectors
   return ga
   
   
   
###########################
##   Sequences
###########################
   
   
def _make_translation_table_for_complementation( ):
   t = [ chr(i) for i in xrange(256) ]
   t[ ord('A') ] = 'T'
   t[ ord('T') ] = 'A'
   t[ ord('C') ] = 'G'
   t[ ord('G') ] = 'C'
   t[ ord('a') ] = 't'
   t[ ord('t') ] = 'a'
   t[ ord('c') ] = 'g'
   t[ ord('g') ] = 'c'
   return ''.join( t )
   
cdef bytes _translation_table_for_complementation = _make_translation_table_for_complementation( )

cpdef bytes reverse_complement( bytes seq ):
   """Returns the reverse complement of DNA sequence 'seq'. Does not yet
   work with extended IUPAC nucleotide letters or RNA."""

   return seq[ ::-1 ].translate( _translation_table_for_complementation )

base_to_column = { 'A': 0, 'C': 1, 'G': 2, 'T': 3, 'N': 4 }

cdef class Sequence( object ):
   """A Sequence, typically of DNA, with a name.
   """
   
   def __init__( self, bytes seq, str name="unnamed" ):
      self.seq = seq
      self.name = name
      self.descr = None
      
   cpdef Sequence get_reverse_complement( self ):
      return Sequence(
         reverse_complement( self.seq ),
         "revcomp_of_" + self.name )
         
   def __str__( self ):
      return self.seq
      
   def __repr__( self ):
      return "<%s object '%s' (length %d)>" % ( 
         self.__class__.__name__, self.name, len( self.seq ) )

   def __len__( self ):
      return len( self.seq )
      
   def __getitem__( self, item ):
      if self.name.endswith( "[part]" ):
         new_name = self.name
      else:
         new_name = self.name + "[part]"
      return Sequence( self.seq[ item ], new_name )
   
   def write_to_fasta_file( self, fasta_file ):
      if self.descr is not None:
         fasta_file.write( ">%s %s\n" % ( self.name, self.descr ) )
      else:
         fasta_file.write( ">%s\n" % self.name )
      i = 0
      while i*70 < len(self.seq):
         fasta_file.write( self.seq[ i*70 : (i+1)*70 ] + "\n" )
         i += 1
         
   cpdef object add_bases_to_count_array( Sequence self, numpy.ndarray count_array_ ):
      
      cdef numpy.ndarray[ numpy.int_t, ndim=2 ] count_array = count_array_
      cdef int seq_length = len( self.seq )

      if numpy.PyArray_DIMS( count_array )[0] < seq_length:
         raise ValueError, "'count_array' too small for sequence."
      if numpy.PyArray_DIMS( count_array )[1] < 5:
         raise ValueError, "'count_array' has too few columns."
      
      cdef numpy.npy_intp i
      cdef char b
      cdef char* seq_cstr = self.seq
      for i in xrange( seq_length ):
         b = seq_cstr[i]
         if b == 'A' or b == 'a':
            count_array[ i, 0 ] += 1
         elif b == 'C' or b == 'c':
            count_array[ i, 1 ] += 1
         elif b == 'G' or b == 'g':
            count_array[ i, 2 ] += 1
         elif b == 'T' or b == 't':
            count_array[ i, 3 ] += 1
         elif b == 'N' or b == 'n' or b == ".":
            count_array[ i, 4 ] += 1
         else:
            raise ValueError, "Illegal base letter encountered."
         
      return None
      
   cpdef Sequence trim_left_end( Sequence self, Sequence pattern, float mismatch_prop = 0. ):
      cdef int seqlen = len( self.seq )
      cdef int patlen = len( pattern.seq )
      cdef int minlen
      if seqlen < patlen:
         minlen = seqlen
      else: 
         minlen = patlen
      cdef char * seq_cstr = self.seq
      cdef char * pat_cstr = pattern.seq
      cdef int match = 0
      cdef int i, j
      cdef int num_mismatches
      for i in xrange( 1, minlen+1 ):
         num_mismatches = 0
         for j in xrange( i ):
            if seq_cstr[ j ] != pat_cstr[ patlen - i + j ]:
               num_mismatches += 1
               if num_mismatches > mismatch_prop * i:
                  break
         else:
            match = i
      return self[ match : seqlen ]

   cpdef Sequence trim_right_end( Sequence self, Sequence pattern, float mismatch_prop = 0. ):
      cdef int seqlen = len( self.seq )
      cdef int patlen = len( pattern.seq )
      cdef int minlen
      if seqlen < patlen:
         minlen = seqlen
      else: 
         minlen = patlen
      cdef char * seq_cstr = self.seq
      cdef char * pat_cstr = pattern.seq
      cdef int match = 0
      cdef int i, j
      cdef int num_mismatches
      for i in xrange( 1, minlen+1 ):
         num_mismatches = 0
         for j in xrange( i ):
            if seq_cstr[ seqlen - i + j ] != pat_cstr[ j ]:
               num_mismatches += 1
               if num_mismatches > mismatch_prop * i:
                  break
         else:
            match = i
      return self[ 0 : seqlen-match ]


cdef class SequenceWithQualities( Sequence ):
   """A Sequence with base-call quality scores.
   It now has property  'qual', an integer NumPy array of Sanger/Phred 
   quality scores of the  base calls.
   """

   def __init__( self, bytes seq, str name, bytes qualstr, str qualscale="phred" ):
      """ Construct a SequenceWithQuality object.
      
        seq       - The actual sequence.
        name      - The sequence name or ID
        qualstr   - The quality string. Must have the same length as seq
        qualscale - The encoding scale of the quality string. Must be one of
                      "phred", "solexa", "solexa-old", or "noquals" )
      """
      Sequence.__init__( self, seq, name )
      if qualscale != "noquals":
         if len( seq ) != len( qualstr ):
            raise ValueError, "'seq' and 'qualstr' do not have the same length."
         self._qualstr = qualstr
      else:
         self._qualstr = b''
      self._qualscale = qualscale
      self._qualarr = None
      self._qualstr_phred = b''

   cdef _fill_qual_arr( SequenceWithQualities self ):
      cdef int seq_len = len( self.seq )
      if self._qualscale == "missing":
         raise ValueError, "Quality string missing."
      if seq_len != len( self._qualstr ):
         raise ValueError, "Quality string has not the same length as sequence."
      cdef numpy.ndarray[ numpy.int_t, ndim=1 ] qualarr = numpy.empty( ( seq_len, ), numpy.int )
      cdef int i
      cdef char * qualstr = self._qualstr
      if self._qualscale == "phred":
         for i in xrange( seq_len ):
            qualarr[i] = qualstr[i] - 33
      elif self._qualscale == "solexa":
         for i in xrange( seq_len ):
            qualarr[i] = qualstr[i] - 64
      elif self._qualscale == "solexa-old":
         for i in xrange( seq_len ):
            qualarr[i] = 10 * math.log10( 1 + 10 ** ( qualstr[i] - 64 ) / 10.0 )
      else:
         raise ValueError, "Illegal quality scale '%s'." % self._qualscale
      self._qualarr = qualarr

   property qual:
      def __get__( self ):
         if self._qualarr is None:
            self._fill_qual_arr()
         return self._qualarr
      def __set__( self, newvalue ):
         if not ( isinstance( newvalue, numpy.ndarray ) and newvalue.dtype == numpy.int ) :
            raise TypeError, "qual can only be assigned a numpy array of type numpy.int"
         if not ( newvalue.shape == ( len(self.seq), ) ) :
            raise TypeError, "assignment to qual with illegal shape"
         self._qualarr = newvalue
         self._qualstr = b""
         self._qualscale = "none"
         self._qualstr_phred = b""
              
   def __repr__( self ):
      return "<%s object '%s'>" % ( self.__class__.__name__, self.name )

   def __getitem__( self, item ):
      if self.name.endswith( "[part]" ):
         new_name = self.name
      else:
         new_name = self.name + "[part]"
      return SequenceWithQualities( 
         self.seq[ item ], new_name, self.qualstr[ item ] )

   @property
   def qualstr( self ):
      cdef int seqlen
      cdef char * qualstr_phred_cstr = self._qualstr_phred
      cdef int i
      cdef numpy.ndarray[ numpy.int_t, ndim=1 ] qual_array
      if qualstr_phred_cstr[0] == 0:
         if self._qualscale == "noquals":
            raise ValueError, "Quality string missing" 
         if self._qualscale == "phred":
            self._qualstr_phred = self._qualstr
         else:
            seqlen = len( self.seq )
            self._qualstr_phred = <bytes>(' ') * seqlen
            qualstr_phred_cstr = self._qualstr_phred
            if self._qualarr is None:
               self._fill_qual_arr()
            qual_array = self._qualarr               
            for i in xrange( seqlen ):
               qualstr_phred_cstr[i] = 33 + qual_array[i]            
      return self._qualstr_phred
      

   def write_to_fastq_file( self, fastq_file ):
      if hasattr( self, "descr" ) and self.descr is not None:
         fastq_file.write( "@%s %s\n" % ( self.name, self.descr ) )
      else:
         fastq_file.write( "@%s\n" % self.name )
      fastq_file.write( self.seq + "\n" )
      fastq_file.write( "+\n" )
      fastq_file.write( self.qualstr + "\n" )

   def get_fastq_str( self, bint convert_to_phred=False ):
      sio = cStringIO.StringIO()
      self.write_to_fastq_file( sio, convert_to_phred )
      return sio.getvalue()

   cpdef SequenceWithQualities get_reverse_complement( self ):
      cdef SequenceWithQualities res
      res = SequenceWithQualities(
         reverse_complement( self.seq ),
         "revcomp_of_" + self.name,
         self._qualstr[::-1],
         self._qualscale )
      if self._qualarr is not None:
         res._qualarr = self._qualarr[::-1]
      return res
      
   cpdef object add_qual_to_count_array( SequenceWithQualities self,
         numpy.ndarray count_array_ ):
      
      cdef numpy.ndarray[ numpy.int_t, ndim=2 ] count_array = count_array_
      if self._qualarr is None:
         self._fill_qual_arr()   
      cdef numpy.ndarray[ numpy.int_t, ndim=1 ] qual_array = self._qualarr

      cdef numpy.npy_intp seq_length = numpy.PyArray_DIMS( qual_array  )[0]
      cdef numpy.npy_intp qual_size  = numpy.PyArray_DIMS( count_array )[1]

      if seq_length > numpy.PyArray_DIMS( count_array )[0]:
         raise ValueError, "'count_array' too small for sequence."
      
      cdef numpy.npy_intp i
      cdef numpy.npy_int q
      for i in xrange( seq_length ):
         q = qual_array[i]
         if( q >= qual_size ):
            raise ValueError, "Too large quality value encountered."
         count_array[ i, q ] += 1
         
      return None
   
   cpdef SequenceWithQualities trim_left_end_with_quals( SequenceWithQualities self, 
         Sequence pattern, int max_mm_qual = 5 ):
      cdef int seqlen = len( self.seq )
      cdef int patlen = len( pattern.seq )
      cdef int minlen
      if seqlen < patlen:
         minlen = seqlen
      else: 
         minlen = patlen
      cdef char * seq_cstr = self.seq
      cdef char * pat_cstr = pattern.seq
      cdef int match = 0
      cdef int i, j
      cdef int sum_mm_qual
      if self._qualarr is None:
         self._fill_qual_arr()   
      cdef numpy.ndarray[ numpy.int_t, ndim=1 ] qual_array = self._qualarr
      for i in xrange( 1, minlen+1 ):
         num_mismatches = 0
         for j in xrange( i ):
            if seq_cstr[ j ] != pat_cstr[ patlen - i + j ]:
               sum_mm_qual += qual_array[ j ]
               if sum_mm_qual > max_mm_qual:
                  break
         else:
            match = i
      return self[ match : seqlen ]

   cpdef SequenceWithQualities trim_right_end_with_quals( SequenceWithQualities self, 
          Sequence pattern, int max_mm_qual = 5 ):
      cdef int seqlen = len( self.seq )
      cdef int patlen = len( pattern.seq )
      cdef int minlen
      if seqlen < patlen:
         minlen = seqlen
      else: 
         minlen = patlen
      cdef char * seq_cstr = self.seq
      cdef char * pat_cstr = pattern.seq
      cdef int match = 0
      cdef int i, j
      cdef int sum_mm_qual
      if self._qualarr is None:
         self._fill_qual_arr()   
      cdef numpy.ndarray[ numpy.int_t, ndim=1 ] qual_array = self._qualarr
      for i in xrange( 1, minlen+1 ):
         sum_mm_qual = 0
         for j in xrange( i ):
            if seq_cstr[ seqlen - i + j ] != pat_cstr[ j ]:
               sum_mm_qual += qual_array[ seqlen - i + j ]
               if sum_mm_qual > max_mm_qual:
                  break
         else:
            match = i
      return self[ 0 : seqlen-match ]

   
###########################
##   Alignment
###########################

cdef class Alignment( object ):

   """Alignment base type:

   An alignment object can be defined in different ways but will always
   provide these attributes:
     read:      a SequenceWithQualities object with the read
     aligned:   whether the read is aligned
     iv:        a GenomicInterval object with the alignment position 
   """
   
   def __init__( self, read, iv ):
      self._read = read
      self.iv = iv
      
   @property
   def read( self ):
      return self._read
   
   def __repr__( self ):
      cdef str s
      if self.paired_end:
         s = "Paired-end read"
      else:
         s = "Read"
      if self.aligned:
         return "<%s object: %s '%s' aligned to %s>" % (
            self.__class__.__name__, s, self.read.name, str(self.iv) )
      else:
         return "<%s object: %s '%s', not aligned>" % (
            self.__class__.__name__, s, self.read.name )
            
   @property
   def paired_end( self ):
      return False
         
   @property
   def aligned( self ):
      """Returns True unless self.iv is None. The latter indicates that
      this record decribes a read for which no alignment was found.
      """
      return self.iv is not None

cdef class AlignmentWithSequenceReversal( Alignment ):

   """Many aligners report the read's sequence in reverse-complemented form
   when it was mapped to the reverse strand. For such alignments, a 
   daughter class of this one should be used.
   
   Then, the read is stored as aligned in the 'read_as_aligned' field,
   and get reverse-complemented back to the sequenced form when the 'read'
   attribute is sequenced.
   """

   def __init__( self, SequenceWithQualities read_as_aligned, GenomicInterval iv ):
      self.read_as_aligned = read_as_aligned      
      self._read_as_sequenced = None
      self.iv = iv
      
   property read:
      def __get__( self ):
         if self._read_as_sequenced is None:
            if (not self.aligned) or self.iv.strand != "-":
               self._read_as_sequenced = self.read_as_aligned
            else:
               self._read_as_sequenced = self.read_as_aligned.get_reverse_complement()
               self._read_as_sequenced.name = self.read_as_aligned.name
         return self._read_as_sequenced      
      #def __set__( self, read ):
      #   self.read_as_aligned = read
      #   self._read_as_sequenced = None
         
         
cdef class BowtieAlignment( AlignmentWithSequenceReversal ):

   """When reading in a Bowtie file, objects of the class BowtieAlignment
   are returned. In addition to the 'read' and 'iv' fields (see Alignment
   class), the fields 'reserved' and 'substitutions' are provided. These 
   contain the content of the respective columns of the Bowtie output 
   
   [A parser for the substitutions field will be added soon.]
   """
   
   cdef public str reserved
   cdef public str substitutions
   
   def __init__( self, bowtie_line ):
      cdef str readId, strand, chrom, position, read, qual
      cdef int positionint
      (readId, strand, chrom, position, read, qual, 
         self.reserved, self.substitutions) = bowtie_line.split( '\t' )
      positionint = int( position )
      AlignmentWithSequenceReversal.__init__( self, 
         SequenceWithQualities( read, readId, qual ),
         GenomicInterval( chrom, positionint, positionint + len(read), strand ) )
      
      
cigar_operation_names = {
   'M': 'matched',
   'I': 'inserted',
   'D': 'deleted',
   'N': 'skipped',
   'S': 'soft-clipped',
   'H': 'hard-clipped',
   'P': 'padded' }

cdef class CigarOperation( object ):
   
   cdef public str type
   cdef public int size
   cdef public GenomicInterval ref_iv
   cdef public int query_from, query_to
   
   def __init__( self, str type_, int size, int rfrom, int rto, int qfrom, 
         int qto, str chrom, str strand, bint check=True ):
      self.type = type_
      self.size = size
      self.ref_iv = GenomicInterval( chrom, rfrom, rto, strand )
      self.query_from = qfrom
      self.query_to = qto
      if check and not self.check():
         raise ValueError, "Inconsistent CIGAR operation."
               
   def __repr__( self ):
      return "< %s: %d base(s) %s on ref iv %s, query iv [%d,%d) >" % (
         self.__class__.__name__, self.size, cigar_operation_names[ self.type ],
         str( self.ref_iv ), self.query_from, self.query_to )
         
   def check( CigarOperation self ):
      cdef int qlen = self.query_to - self.query_from
      cdef int rlen = self.ref_iv.length
      if self.type == 'M':
         if not ( qlen == self.size and rlen == self.size ):
            return False
      elif self.type == 'I' or self.type == 'S':
         if not ( qlen == self.size and rlen == 0 ):
            return False
      elif self.type == 'D' or self.type == 'N':
         if not ( qlen == 0 and rlen == self.size ):
            return False
      elif self.type == 'H' or self.type == 'P':
         if not ( qlen == 0 and rlen == 0 ):
            return False
      else:
         return False
      return True

_re_cigar_codes = re.compile( '([A-Z])' )

cpdef list parse_cigar( str cigar_string, int ref_left = 0, str chrom = "", str strand = "." ):
   cdef list split_cigar, cl
   cdef int size
   cdef str code
   split_cigar = _re_cigar_codes.split( cigar_string )
   if split_cigar[-1] != '' or len(split_cigar) % 2 != 1:
      raise ValueError, "Illegal CIGAR string '%s'" % cigar_string
   cl = []
   for i in xrange( len(split_cigar) // 2 ):
      try:
         size = int( split_cigar[2*i] )
      except ValueError:
         raise ValueError, "Illegal CIGAR string '%s'" % cigar_string
      code  = split_cigar[2*i+1]
      cl.append( ( code, size ) )
   return build_cigar_list( cl, ref_left, chrom, strand  )

cpdef list build_cigar_list( list cigar_pairs, int ref_left = 0, str chrom = "", str strand = "." ):
   cdef list split_cigar, res
   cdef int rpos, qpos, size
   cdef str code
   rpos = ref_left
   qpos = 0
   res = []
   for code, size in cigar_pairs:
      if code == 'M':
         res.append( CigarOperation ( 
            'M', size, rpos, rpos + size, qpos, qpos + size, chrom, strand ) )
         rpos += size
         qpos += size
      elif code == 'I':
         res.append( CigarOperation ( 
            'I', size, rpos, rpos, qpos, qpos + size, chrom, strand ) )
         qpos += size
      elif code == 'D':
         res.append( CigarOperation ( 
            'D', size, rpos, rpos + size, qpos, qpos, chrom, strand ) )
         rpos += size
      elif code == 'N':
         res.append( CigarOperation ( 
            'N', size, rpos, rpos + size, qpos, qpos, chrom, strand ) )
         rpos += size
      elif code == 'S':
         res.append( CigarOperation ( 
            'S', size, rpos, rpos, qpos, qpos + size, chrom, strand ) )
         qpos += size
      elif code == 'H':
         res.append( CigarOperation ( 
            'H', size, rpos, rpos, qpos, qpos, chrom, strand ) )
      elif code == 'P':
         res.append( CigarOperation ( 
            'P', size, rpos, rpos, qpos, qpos, chrom, strand ) )
      else:
         raise ValueError, "Unknown CIGAR code '%s' encountered." % code
   return res
      
cdef _parse_SAM_optional_field_value( str field ):
   if len(field) < 5 or field[2] != ':' or field[4] != ':':
      raise ValueError, "Malformatted SAM optional field '%'" % field
   if field[3] == 'A':
      return field[5]
   elif field[3] == 'i':
      return int( field[5:] )
   elif field[3] == 'f':
      return float( field[5:] )
   elif field[3] == 'Z':
      return field[5:]
   elif field[3] == 'H':
      return int( field[5:], 16 )
   else:
      raise ValueError, "SAM optional field with illegal type letter '%s'" % field[2]

cigar_operation_codes = [ 'M', 'I', 'D', 'N', 'S', 'H', 'P', '=', 'X']
cigar_operation_code_dict = dict( [ (x,i) for i,x in enumerate( cigar_operation_codes ) ] )

cdef class SAM_Alignment( AlignmentWithSequenceReversal ):

   """When reading in a SAM file, objects of the class SAM_Alignment
   are returned. In addition to the 'read', 'iv' and 'aligned' fields (see 
   Alignment class), the following fields are provided:
    - aQual: the alignment quality score
    - cigar: a list of CigarOperatio objects, describing the alignment
    - tags: the extra information tags [not yet implemented]
   """

   def to_pysam_AlignedRead( self, sf ):
      try:
         import pysam
      except ImportError:
         sys.stderr.write( "Please Install PySam to use this functionality (http://code.google.com/p/pysam/)" )
         raise
      
      a = pysam.AlignedRead()
      a.seq = self.read.seq
      a.qual = self.read.qualstr
      a.qname = self.read.name
      a.flag = self.flag
      a.tags = self.optional_fields
      if self.aligned:
         a.cigar = [ (cigar_operation_code_dict[c.type], c.size) for c in self.cigar ]
         a.pos = self.iv.start
         a.tid = sf.gettid( self.iv.chrom )
         a.isize = self.inferred_insert_size
         a.mapq = self.aQual
      else:
         a.pos  = -1
         a.tid  = -1
      if self.mate_aligned:
         a.mrnm = sf.gettid( self.mate_start.chrom )
         a.mpos = self.mate_start.start
      else:
         a.mrnm = -1
         a.mpos = -1
      return a

   @classmethod
   def from_pysam_AlignedRead( cls, read, samfile ):
      strand = "-" if read.is_reverse else "+"
      if not read.is_unmapped:
          chrom = samfile.getrname(read.tid)
          iv = GenomicInterval( chrom, read.pos, read.aend, strand )
      else:
          iv = None
      if read.qual != "*":
         seq = SequenceWithQualities( read.seq, read.qname, read.qual )
      else:
         seq = SequenceWithQualities( read.seq, read.qname, read.qual, "noquals" )
      a = SAM_Alignment( seq, iv )
      a.cigar = build_cigar_list( [ (cigar_operation_codes[code], length) for (code, length) in read.cigar ] , read.pos, chrom, strand ) if iv != None else []
      a.inferred_insert_size = read.isize
      a.aQual = read.mapq
      a.flag = read.flag
      a.proper_pair = read.is_proper_pair
      a.not_primary_alignment = read.is_secondary
      a.failed_platform_qc = read.is_qcfail
      a.pcr_or_optical_duplicate = read.is_duplicate
      a.original_sam_line = ""
      a.optional_fields = read.tags
      if read.is_paired:
         if not read.mate_is_unmapped:
            strand = "-" if read.mate_is_reverse else "+"
            a.mate_start = GenomicPosition( samfile.getrname(read.mrnm), read.mpos, strand )
         else:
            a.mate_start = None
         if read.is_read1:
            a.pe_which = intern( "first" )
         elif read.is_read2:  
            a.pe_which = intern( "second" )
         else:
            a.pe_which = intern( "unknown" )
      else:
         a.pe_which = intern( "not_paired_end" )
      return a
         
   @classmethod
   def from_SAM_line( cls, line ):
      cdef str qname, flag, rname, pos, mapq, cigar, 
      cdef str mrnm, mpos, isize, seq, qual
      cdef list optional_fields
      cdef int posint, flagint
      cdef str strand
      cdef list cigarlist
      cdef SequenceWithQualities swq
            
      fields = line.rstrip().split( "\t" )
      if len( fields ) < 10:
         raise ValueError, "SAM line does not contain at least 11 tab-delimited fields."
      (qname, flag, rname, pos, mapq, cigar, mrnm, mpos, isize, 
         seq, qual) = fields[ 0:11 ]
      optional_fields = fields[ 11: ]      
      
      if seq.count( "=" ) > 0:
         raise ValueError, "Sequence in SAM file contains '=', which is not supported."
      if seq.count( "." ) > 0:
         raise ValueError, "Sequence in SAM file contains '.', which is not supported."
      flagint = int( flag )
      
      if flagint & 0x0004:     # flag "query sequence is unmapped" 
         iv = None
         cigarlist = None
         if rname != "*":     # flag "query sequence is unmapped"      
            warnings.warn( "Malformed SAM line: RNAME != '*' although flag bit &0x0004 set" )
      else:
         if rname == "*":
            raise ValueError, "Malformed SAM line: RNAME == '*' although flag bit &0x0004 cleared"
         posint = int( pos ) - 1   # SAM is one-based, but HTSeq is zero-based!
         if flagint & 0x0010:      # flag "strand of the query"
            strand = "-"
         else:
            strand = "+"
         cigarlist = parse_cigar( cigar, posint, rname, strand )
         iv = GenomicInterval( rname, posint, cigarlist[-1].ref_iv.end, strand )   
            
      if qual != "*":
         swq = SequenceWithQualities( seq.upper(), qname, qual )
      else:
         swq = SequenceWithQualities( seq.upper(), qname, "", "noquals" )

      alnmt = SAM_Alignment( swq, iv )
      alnmt.flag = flagint   
      alnmt.cigar = cigarlist
      alnmt.optional_fields = [ ( field[:2], _parse_SAM_optional_field_value( field ) ) for field in optional_fields ]
      alnmt.aQual = int( mapq )
      alnmt.inferred_insert_size = int( isize )
      alnmt.original_sam_line = line
      
      if flagint & 0x0001:         # flag "read is paired in sequencing"
         if flagint & 0x0008:      # flag "mate is unmapped"
            if mrnm != "*":
               warnings.warn( "Malformed SAM line: MRNM != '*' although flag bit &0x0008 set" )
            alnmt.mate_start = None
         else:
            if mrnm == "*":
               raise ValueError, "Malformed SAM line: MRNM == '*' although flag bit &0x0008 cleared"               
            posint = int( mpos ) - 1
            if flagint & 0x0020:   # flag "strand of the mate"
               strand = "-"
            else:
               strand = "+"           
            alnmt.mate_start = GenomicPosition( mrnm, posint, strand )   
            if alnmt.mate_start.chrom == "=":
               if alnmt.iv is not None:
                  alnmt.mate_start.chrom = alnmt.iv.chrom
               else:
                  warnings.warn( "Malformed SAM line: MRNM == '=' although read is not aligned." )
         if flagint & 0x0040:
            alnmt.pe_which = intern( "first" )
         elif flagint & 0x0080:
            alnmt.pe_which = intern( "second" )
         else:
            alnmt.pe_which = intern( "unknown" )
      else:
         alnmt.mate_start = None
         alnmt.pe_which = intern( "not_paired_end" )
        
      alnmt.proper_pair = flagint & 0x0002 > 0
      alnmt.not_primary_alignment = flagint & 0x0100 > 0
      alnmt.failed_platform_qc = flagint & 0x0200 > 0
      alnmt.pcr_or_optical_duplicate = flagint & 0x0400 > 0

      return alnmt

   property flag:
      def __get__( self ):
         return self._flag
      def __set__( self, value ):
         self._flag = value
   
   @property
   def paired_end( self ):
      return self.pe_which != "not_paired_end"
   
   @property
   def mate_aligned( self ):
      return self.mate_start is not None
   
   def get_sam_line( self ):
      cdef str cigar = ""
      cdef GenomicInterval query_start, mate_start
      cdef CigarOperation cop
             
      if self.aligned:
         query_start = self.iv
      else:
         query_start = GenomicPosition( "*", -1 )
          
      if self.mate_start is not None:
         mate_start = self.mate_start
      else:
         mate_start = GenomicPosition( "*", -1 )
                
      if self.cigar is not None:
         for cop in self.cigar:
            cigar += str(cop.size) + cop.type
      else:
         cigar = "*"
       
      return '\t'.join( ( self.read.name, str(self.flag), query_start.chrom, 
          str(query_start.start+1), str(self.aQual), cigar, mate_start.chrom, 
          str(mate_start.pos+1), str(self.inferred_insert_size), 
           self.read_as_aligned.seq, self.read_as_aligned.qualstr,
           '\t'.join( self.raw_optional_fields() ) ) )      

   def optional_field( SAM_Alignment self, str tag ):
      res = [ p for p in self.optional_fields if p[0] == tag ]
      if len(res) == 1:
         return res[0][1]
      else:
         if len(res) == 0:
            raise KeyError, "SAM optional field tag %s not found" % tag
         else:
            raise ValueError, "SAM optional field tag %s not unique" % tag
   
   def raw_optional_fields( self ):
      res = []
      for op in self.optional_fields:
         if op[1].__class__ == str:
            if len(op[1]) == 1:
               tc = "A"
            else:
               tc = "Z"
         elif op[1].__class__ == int:
            tc = "i"
         elif op[1].__class__ == float:
            tc = "j"
         else:
            tc = "H"
         res.append( ":".join( [ op[0], tc, str(op[1]) ] ) )
      return res


###########################
##   Helpers
###########################

cpdef list quotesafe_split( bytes s, bytes split=b';', bytes quote=b'"' ):
   cdef list l = []
   cdef int i = 0
   cdef int begin_token = 0
   cdef bint in_quote = False
   cdef char * s_c = s
   cdef char split_c = split[0]
   cdef char quote_c = quote[0]
   if len(split) != 1:
      raise ValueError, "'split' must be length 1"
   if len(quote) != 1:
      raise ValueError, "'quote' must be length 1"
   while s_c[i] != 0:
      if s_c[i] == quote_c:
         in_quote = not in_quote
      elif (not in_quote) and s_c[i] == split_c:
         l.append( s[ begin_token : i ] )
         begin_token = i + 1
      i += 1
   l.append( s[ begin_token : ] )
   if in_quote:
      raise ValueError, "unmatched quote"
   return l



cimport numpy

cdef class GenomicInterval:

   cdef public str chrom
   cdef public long start
   cdef public long end
   cdef str _strand

   cpdef is_contained_in( self, GenomicInterval iv )
   cpdef contains( self, GenomicInterval iv )
   cpdef overlaps( self, GenomicInterval iv )
   cpdef extend_to_include( GenomicInterval self, GenomicInterval iv )

cdef class GenomicPosition( GenomicInterval ):
   pass


cdef class Sequence( object ):
   cdef public bytes seq
   cdef public str name
   cdef public str descr
   cpdef Sequence get_reverse_complement( self )      
   cpdef object add_bases_to_count_array( Sequence self, numpy.ndarray count_array_ )
   cpdef Sequence trim_left_end( Sequence self, Sequence pattern, float mismatch_prop = ? )   
   cpdef Sequence trim_right_end( Sequence self, Sequence pattern, float mismatch_prop = ? )   
   

cdef class SequenceWithQualities( Sequence ):
   cdef readonly bytes _qualstr
   cdef readonly bytes _qualstr_phred
   cdef readonly str _qualscale
   cdef readonly object _qualarr
   cdef _fill_qual_arr( SequenceWithQualities self )
   cpdef object add_qual_to_count_array( SequenceWithQualities self, numpy.ndarray count_array_ )
   cpdef SequenceWithQualities trim_left_end_with_quals( SequenceWithQualities self, 
         Sequence pattern, int max_mm_qual = ? )
   cpdef SequenceWithQualities trim_right_end_with_quals( SequenceWithQualities self, 
         Sequence pattern, int max_mm_qual = ? )

      
cdef class Alignment( object ):
   cdef public SequenceWithQualities _read
   cdef public GenomicInterval iv
   
cdef class AlignmentWithSequenceReversal( Alignment ):   
   cdef public SequenceWithQualities read_as_aligned
   cdef public SequenceWithQualities _read_as_sequenced

cdef class SAM_Alignment( AlignmentWithSequenceReversal ):
   cdef public list cigar
   cdef public int aQual
   cdef public GenomicPosition mate_start
   cdef public str pe_which
   cdef public int inferred_insert_size
   cdef public bint proper_pair
   cdef public bint not_primary_alignment
   cdef public bint failed_platform_qc
   cdef public bint pcr_or_optical_duplicate
   cdef readonly str original_sam_line
   cdef int _flag
   cdef public list optional_fields




import HTSeq
import numpy
from matplotlib import pyplot

bamfile = HTSeq.BAM_Reader( "SRR001432_head.bam" )
gtffile = HTSeq.GFF_Reader( "Homo_sapiens.GRCh37.56_chrom1.gtf" )
halfwinwidth = 3000
fragmentsize = 200

coverage = HTSeq.GenomicArray( "auto", stranded=False, typecode="i" )
for almnt in bamfile:
   if almnt.aligned:
      almnt.iv.length = fragmentsize
      coverage[ almnt.iv ] += 1

tsspos = set()
for feature in gtffile:
   if feature.type == "exon" and feature.attr["exon_number"] == "1":
      tsspos.add( feature.iv.start_d_as_pos )

profile = numpy.zeros( 2*halfwinwidth, dtype='i' )      
for p in tsspos:
   window = HTSeq.GenomicInterval( p.chrom, p.pos - halfwinwidth, p.pos + halfwinwidth, "." )
   wincvg = numpy.fromiter( coverage[window], dtype='i', count=2*halfwinwidth )
   if p.strand == "+":
      profile += wincvg
   else:
      profile += wincvg[::-1]




"""HTSeq is a package to process high-throughput sequencing data.

See http://www-huber.embl.de/users/anders/HTSeq for documentation.
"""

import itertools, warnings, os, shlex

try:
   from _HTSeq import *
except ImportError:
   if os.path.isfile( "setup.py" ):
      raise ImportError( "Cannot import 'HTSeq' when working directory is HTSeq's own build directory.")
   else:
      raise
      
from _version import __version__

#from vcf_reader import *

#########################
## Utils
#########################


class FileOrSequence( object ):
   """ The construcutor takes one argument, which may either be a string,
   which is interpreted as a file name (possibly with path), or a
   connection, by which we mean a text file opened for reading, or 
   any other object that can provide an iterator over strings 
   (lines of the file).
   
   The advantage of passing a file name instead of an already opened file
   is that if an iterator is requested several times, the file will be
   re-opened each time. If the file is already open, its lines can be read
   only once, and then, the iterator stays exhausted.      
   
   Furthermore, if a file name is passed that end in ".gz" or ".gzip"
   (case insensitive), it is transparently gunzipped.
   """
   
   def __init__( self, filename_or_sequence ):      
      self.fos = filename_or_sequence
      self.line_no = None
      
   def __iter__( self ):
      self.line_no = 1
      if isinstance( self.fos, str ):
         if self.fos.lower().endswith( ( ".gz" , ".gzip" ) ):
            lines = gzip.open( self.fos )
         else:
            lines = open( self.fos )
      else:
         lines = self.fos
      for line in lines:
         yield line
         self.line_no += 1
      if isinstance( self.fos, str ):
         lines.close()
      self.line_no = None
         
   def __repr__( self ):
      if isinstance( self.fos, str ):
         return "<%s object, connected to file name '%s'>" % (
            self.__class__.__name__, self.fos )
      else:
         return "<%s object, connected to %s >" % (
            self.__class__.__name__, repr( self.fos ) )   
            
   def get_line_number_string( self ):
      if self.line_no is None:
         if isinstance( self.fos, str ):
            return "file %s closed" % self.fos
         else:
            return "file closed"
      if isinstance( self.fos, str ):
         return "line %d of file %s" % ( self.line_no, self.fos )
      else:
         return "line %d" % self.line_no
         

#########################
## Features
#########################

class GenomicFeature( object ):
   
   """A genomic feature, i.e., an interval on a genome with metadata.
   
   At minimum, the following information should be provided by slots:
   
     name: a string identifying the feature (e.g., a gene symbol)
     type: a string giving the feature type (e.g., "gene", "exon")
     iv: a GenomicInterval object specifying the feature locus     
   """
   
   def __init__( self, name, type_, interval ):
      self.name = name
      self.type = intern( type_ )
      self.iv = interval
      
   def __repr__( self ):
      return "<%s: %s '%s' at %s: %d -> %d (strand '%s')>" % \
         ( self.__class__.__name__, self.type, self.name, 
         self.iv.chrom, self.iv.start_d, self.iv.end_d, self.iv.strand )
         
   def __eq__( self, other ):
      if not isinstance( other, GenomicFeature ):
         return False
      return self.name == other.name and self.type == other.type and \
         self.iv == other.iv
         
   def __neq__( self, other ):
      if not isinstance( other, GenomicFeature ):
         return True
      return not self.__eq__( other )
   
   def get_gff_line( self, with_equal_sign=False ):
      try:
         source = self.source
      except AttributeError:
         source = "."
      try:
         score = self.score
      except AttributeError:
         score = "."
      try:
         frame = self.frame
      except AttributeError:
         frame = "."
      try:
         attr = self.attr
      except AttributeError:
         attr = { 'ID': self.name }
      if with_equal_sign:
         sep = "="
      else:
         sep = " "
      attr_str = '; '.join( [ '%s%s\"%s\"' % ( ak, sep, attr[ak] ) for ak in attr ] )
      return "\t".join( str(a) for a in ( self.iv.chrom, source, 
         self.type, self.iv.start+1, self.iv.end, score, 
         self.iv.strand, frame, attr_str ) ) + "\n"
         

_re_attr_main = re.compile( "\s*([^\s\=]+)[\s=]+(.*)" )
_re_attr_empty = re.compile( "^\s*$" )

def parse_GFF_attribute_string( attrStr, extra_return_first_value=False ):
   """Parses a GFF attribute string and returns it as a dictionary.
   
   If 'extra_return_first_value' is set, a pair is returned: the dictionary
   and the value of the first attribute. This might be useful if this is the ID.
   """
   if attrStr.endswith( "\n" ):
      attrStr = attrStr[:-1]
   d = {}
   first_val = "_unnamed_"
   for (i, attr) in itertools.izip( itertools.count(), _HTSeq.quotesafe_split( attrStr ) ):
      if _re_attr_empty.match( attr ):
         continue
      if attr.count( '"' ) not in ( 0, 2 ):
         raise ValueError, "The attribute string seems to contain mismatched quotes."
      mo = _re_attr_main.match( attr )
      if not mo:
         raise ValueError, "Failure parsing GFF attribute line"
      val = mo.group(2)
      if val.startswith( '"' ) and val.endswith( '"' ):
         val = val[1:-1]
      #val = urllib.unquote( val )
      d[ intern(mo.group(1)) ] = intern(val)
      if extra_return_first_value and i == 0:
         first_val = val         
   if extra_return_first_value:
      return ( d, first_val )
   else:
      return d

_re_gff_meta_comment = re.compile( "##\s*(\S+)\s+(\S*)" )

class GFF_Reader( FileOrSequence ):

   """Parse a GFF file
   
   Pass the constructor either a file name or an iterator of lines of a 
   GFF files. If a file name is specified, it may refer to a gzip compressed
   file.
   
   Iterating over the object then yields GenomicFeature objects.
   """
   
   def __init__( self, filename_or_sequence, end_included=True ):
      FileOrSequence.__init__( self, filename_or_sequence )
      self.end_included = end_included
      self.metadata = {}
   
   
   def __iter__( self ):
      for line in FileOrSequence.__iter__( self ):
         if line == "\n":
            continue
         if line.startswith( '#' ):
            if line.startswith( "##" ):
               mo = _re_gff_meta_comment.match( line )
               if mo:
                  self.metadata[ mo.group(1) ] = mo.group(2)
            continue
         ( seqname, source, feature, start, end, score, 
            strand, frame, attributeStr ) = line.split( "\t", 8 )   
         ( attr, name ) = parse_GFF_attribute_string( attributeStr, True )
         if self.end_included:
            iv = GenomicInterval( seqname, int(start)-1, int(end), strand )
         else:
            iv = GenomicInterval( seqname, int(start)-1, int(end)-1, strand )
         f = GenomicFeature( name, feature, iv )
         if score != ".":
            score = float( score )
         if frame != ".":
            frame = int( frame )
         f.source = source
         f.score = score
         f.frame = frame
         f.attr = attr
         yield f

def make_feature_dict( feature_sequence ):
   """A feature dict is a convenient way to organize a sequence of Feature 
   object (which you have got, e.g., from parse_GFF).
   
   The function returns a dict with all the feature types as keys. Each value
   of this dict is again a dict, now of feature names. The values of this dict
   is a list of feature.
   
   An example makes this clear. Let's say you load the C. elegans GTF file
   from Ensemble and make a feature dict:
   
   >>> worm_features_dict = HTSeq.make_feature_dict( HTSeq.parse_GFF( 
   ...     "test_data/Caenorhabditis_elegans.WS200.55.gtf.gz" ) )

   (This command may take a few minutes to deal with the 430,000 features
   in the GTF file. Note that you may need a lot of RAM if you have millions
   of features.)

   Then, you can simply access, say, exon 0 of gene "F08E10.4" as follows:
   >>> worm_features_dict[ 'exon' ][ 'F08E10.4' ][ 0 ]
   <GenomicFeature: exon 'F08E10.4' at V: 17479353 -> 17479001 (strand '-')>
   """
   
   res = {}
   for f in feature_sequence:
      if f.type not in res:
         res[ f.type ] = {}
      res_ftype = res[ f.type ]
      if f.name not in res_ftype:
         res_ftype[ f.name ] = [ f ]
      else:
         res_ftype[ f.name ].append( f )
   return res



#########################
## GenomicArray
#########################

def read_chrom_lens( filename, delimiter="\t" ):
   return dict( ( ( chrom, int(len) ) 
      for chrom, len in csv.reader( open(filename), delimiter=delimiter ) ) )
            

#########################
## Sequence readers
#########################

_re_fasta_header_line = re.compile( r'>\s*(\S+)\s*(.*)' )      

class FastaReader( FileOrSequence ):
   """A Fasta_Reader is associated with a FASTA file or an open connection
   to a file-like object with content in FASTA format.
   It can generate an iterator over the sequences.
   """
   
   def __iter__( self ):
      seq = None
      for line in FileOrSequence.__iter__( self ):
         if line.startswith( ">" ):
            if seq:
               s = Sequence( seq, name )
               s.descr = descr
               yield s
            mo = _re_fasta_header_line.match( line )
            name = mo.group(1)
            descr = mo.group(2)
            seq = ""
         else: 
            assert seq is not None, "FASTA file does not start with '>'."
            seq += line[:-1]
      if seq is not None:
         s = Sequence( seq, name )
         s.descr = descr
         yield s
         
   def get_sequence_lengths( self ):   
      seqname = None
      seqlengths = {}
      for line in FileOrSequence.__iter__( self ):
         if line.startswith( ">" ):
            if seqname is not None:
               seqlengths[ seqname ] = length
            mo = _re_fasta_header_line.match( line )
            seqname = mo.group(1)
            length = 0
         else: 
            assert seqname is not None, "FASTA file does not start with '>'."
            length += len( line.rstrip() )
      if seqname is not None:
         seqlengths[ seqname ] = length
      return seqlengths
      
   @staticmethod   
   def _import_pysam():
      global pysam
      try:
         import pysam
      except ImportError:
         sys.stderr.write( "Please install the 'pysam' package to be able to use the Fasta indexing functionality." )
         raise
      
   def build_index( self, force = False ):
      self._import_pysam()
      if not isinstance( self.fos, str ):
         raise TypeError, "This function only works with FastaReader objects " + \
            "connected to a fasta file via file name"
      index_filename = self.fos + ".fai"
      if os.access( index_filename, os.R_OK ):
         if (not force) and os.stat( self.filename_or_sequence ).st_mtime <= \
               os.stat( index_filename ).st_mtime:
            # index is up to date
            return
      pysam.faidx( self.fos )
      if not os.access( index_filename, os.R_OK ):
         raise SystemError, "Building of Fasta index failed due to unknown error."
      
   def __getitem__( self, iv ):
      if not isinstance( iv, GenomicInterval ):
         raise TypeError, "GenomicInterval expected as key."
      if not isinstance( self.fos, str ):
         raise TypeError, "This function only works with FastaReader objects " + \
            "connected to a fasta file via file name"
      self._import_pysam()
      fasta = pysam.faidx( self.fos, "%s:%d-%d" % ( iv.chrom, iv.start, iv.end-1 ) )
      ans = list( FastaReader( fasta ) )
      assert len( ans ) == 1
      ans[0].name = str(iv)
      if iv.strand != "-":
         return ans[0]
      else:
         return ans[0].get_reverse_complement()
            
class FastqReader( FileOrSequence ):
   """A Fastq object is associated with a FASTQ self.file. When an iterator
   is requested from the object, the FASTQ file is read.
   
   qual_scale is one of "phred", "solexa", "solexa-old".
   """

   def __init__( self, file_, qual_scale = "phred" ):
      FileOrSequence.__init__( self, file_ )
      self.qual_scale = qual_scale
      if qual_scale not in ( "phred", "solexa", "solexa-old" ):
         raise ValueError, "Illegal quality scale."
      
   def __iter__( self ):
      fin = FileOrSequence.__iter__( self )
      while True:
         id1  = fin.next()
         seq  = fin.next()
         id2  = fin.next()
         qual = fin.next()
         if qual == "":
            if id1 != "":
               warnings.warn( "Number of lines in FASTQ file is not "
                  "a multiple of 4. Discarding the last, "
                  "incomplete record" )
            break
            
         if not qual.endswith( "\n" ):
            qual += "\n"
         if not id1.startswith( "@" ):
            raise ValueError( "Primary ID line in FASTQ file does"
               "not start with '@'. Either this is not FASTQ data or the parser got out of sync." )
         if not id2.startswith( "+" ):
            raise ValueError( "Secondary ID line in FASTQ file does"
               "not start with '+'. Maybe got out of sync." )
         if len( id2 ) > 2 and id1[1:] != id2[1:]:
            raise ValueError( "Primary and secondary ID line in FASTQ"
               "disagree." )
               
         yield SequenceWithQualities( seq[:-1], id1[1:-1], qual[:-1], 
            self.qual_scale )

class BowtieReader( FileOrSequence ):
   """A BowtieFile object is associated with a Bowtie output file that 
   contains short read alignments. It can generate an iterator of Alignment
   objects."""

   def __iter__( self ):
      for line in FileOrSequence.__iter__( self ):
         try:
            algnt = BowtieAlignment( line )
         except ValueError:
            if line.startswith( "Reported " ):
               continue
            warnings.warn( "BowtieReader: Ignoring the following line, which could not be parsed:\n%s\n" % line,
               RuntimeWarning )
         yield algnt
   
def bundle_multiple_alignments( sequence_of_alignments ):      
   """Some alignment programs, e.g., Bowtie, can output multiple alignments,
   i.e., the same read is reported consecutively with different alignments.
   This function takes an iterator over alignments and bundles consecutive
   alignments regarding the same read to a list of Alignment objects and
   returns an iterator over these.
   """
   alignment_iter = iter( sequence_of_alignments )
   algnt = alignment_iter.next()
   ma = [ algnt ]
   for algnt in alignment_iter:
      if algnt.read.name != ma[0].read.name:
         yield ma
         ma = [ algnt ]
      else:
         ma.append( algnt )
   yield ma   
   
  
class SolexaExportAlignment( Alignment ):
   """Iterating over SolexaExportReader objects will yield SoelxaExportRecord
   objects. These have four fields:
      read          - a SequenceWithQualities object 
      aligned       - a boolean, indicating whether the object was aligned
      iv            - a GenomicInterval giving the alignment (or None, if not aligned)
      passed_filter - a boolean, indicating whether the object passed the filter
      nomatch_code  - a code indicating why no match was found (or None, if the
        read was aligned)
        
   As long as 'aligned' is True, a SolexaExportRecord can be treated as an 
   Alignment object.
   """
   
   def __init__( self ):
      # Data is filled in by SolexaExportRecord
      pass
   
   def __repr__( self ):
      if self.aligned:
         return "< %s object: Read '%s', aligned to %s >" % (
            self.__class__.__name__, self.read.name, self.iv )
      else:
         return "< %s object: Non-aligned read '%s' >" % ( 
            self.__class__.__name__, self.read.name )

class SolexaExportReader( FileOrSequence ):
   """Parser for *_export.txt files from the SolexaPipeline software.
   
   Iterating over a SolexaExportReader yields SolexaExportRecord objects.
   """

   def __init__( self, filename_or_sequence, solexa_old = False ):
      FileOrSequence.__init__( self, filename_or_sequence)
      if solexa_old:
         self.qualscale = "solexa-old"
      else:
         self.qualscale = "solexa"

   @classmethod
   def parse_line_bare( dummy, line ):
      if line[-1] == "\n":
         line = line[:-1]
      res = {}
      ( res['machine'], res['run_number'], res['lane'], res['tile'], res['x_coord'], 
         res['y_coord'], res['index_string'], res['read_nbr'], res['read_seq'], 
         res['qual_str'], res['chrom'], res['contig'], res['pos'], res['strand'], 
         res['match_descr'], res['single_read_algnt_score'], 
         res['paired_read_algnt_score'], res['partner_chrom'], res['partner_contig'], 
         res['partner_offset'], res['partner_strand'], res['passed_filtering'] ) \
         = line.split( "\t" )
      return res

   def __iter__( self ):   
      for line in FileOrSequence.__iter__( self ):
         record = SolexaExportAlignment()
         fields = SolexaExportReader.parse_line_bare( line )
         if fields['read_nbr'] != "1":
            warnings.warn( "Paired-end read encountered. PE is so far supported only for " +
               "SAM files, not yet for SolexaExport. All PE-related fields are ignored. " )
         record.read = SequenceWithQualities( 
            fields['read_seq'], 
            "%s:%s:%s:%s:%s#0" % (fields['machine'], fields['lane'], fields['tile'], 
               fields['x_coord'], fields['y_coord'] ),
            fields['qual_str'], self.qualscale )
         if fields['passed_filtering'] == 'Y':
            record.passed_filter = True
         elif fields['passed_filtering'] == 'N':
            record.passed_filter = False
         else:
            raise ValueError, "Illegal 'passed filter' value in Solexa export data: '%s'." % fields['passed_filtering']
         record.index_string = fields['index_string']
         if fields['pos'] == '':
            record.iv = None
            record.nomatch_code = fields['chrom']
         else:
            if fields['strand'] == 'F':
               strand = '+'
            elif fields['strand'] == 'R':
               strand = '-'
            else:
               raise ValueError, "Illegal strand value in Solexa export data."
            start = int( fields['pos'] )
            chrom = fields['chrom']
            if fields['chrom'] == "":
               chrom = fields['contig']
            record.iv = GenomicInterval( chrom, start, 
               start + len( fields['read_seq'] ), strand )
         yield record
 
class SAM_Reader( FileOrSequence ):
   """A SAM_Reader object is associated with a SAM file that 
   contains short read alignments. It can generate an iterator of Alignment
   objects."""

   def __iter__( self ):
      for line in FileOrSequence.__iter__( self ):
         if line.startswith( "@" ):
            # do something with the header line
            continue
         try:
            algnt = SAM_Alignment.from_SAM_line( line )
         except ValueError, e:
            e.args = e.args + ( self.get_line_number_string(), )
            raise
         yield algnt
       
class GenomicArrayOfSets( GenomicArray ):

   """A GenomicArrayOfSets is a specialization of GenomicArray that allows to store
   sets of objects. On construction, the step vectors are initialized with empty sets.
   By using the 'add_value' method, objects can be added to intervals. If an object
   is already present in the set(s) at this interval, an the new object is added to
   the present set, and the set is split if necessary.
   """

   def __init__( self, chroms, stranded=True, storage='step', memmap_dir = "" ):
      GenomicArray.__init__( self, chroms, stranded, 'O', storage, memmap_dir )

   def add_chrom( self, chrom, length = sys.maxint, start_index = 0 ):
      GenomicArray.add_chrom( self, chrom, length, start_index )
      for cv in self.chrom_vectors[ chrom ].values():
         cv[:] = set()
         cv.is_vector_of_sets = True
      
       
###########################
##   paired-end handling
###########################
                
                
def pair_SAM_alignments( alignments, bundle=False ):

   mate_missing_count = [0]

   def process_list( almnt_list ):
      while len( almnt_list ) > 0:
         a1 = almnt_list.pop( 0 )
         # Find its mate
         for a2 in almnt_list:
            if a1.pe_which == a2.pe_which:
               continue
            if a1.aligned != a2.mate_aligned or a1.mate_aligned != a2.aligned:
               continue
            if not (a1.aligned and a2.aligned):
               break
            if a1.iv.chrom == a2.mate_start.chrom and a1.iv.start == a2.mate_start.pos and \
                  a2.iv.chrom == a1.mate_start.chrom and a2.iv.start == a1.mate_start.pos:
               break
         else:
            if a1.mate_aligned:
               mate_missing_count[0] += 1
               if mate_missing_count[0] == 1:
                  warnings.warn( "Read " + a1.read.name + " claims to have an aligned mate " +
                     "which could not be found in an adjacent line." )
            a2 = None
         if a2 is not None:
            almnt_list.remove( a2 )
         if a1.pe_which == "first":
            yield ( a1, a2 )
         else:
            assert a1.pe_which == "second"
            yield ( a2, a1 )

   almnt_list = []
   current_name = None
   for almnt in alignments:
      if not almnt.paired_end:
         raise ValueError, "'pair_alignments' needs a sequence of paired-end alignments"
      if almnt.pe_which == "unknown":
         raise ValueError, "Paired-end read found with 'unknown' 'pe_which' status."
      if almnt.read.name == current_name:
         almnt_list.append( almnt )
      else:
         if bundle:
            yield list( process_list( almnt_list ) )
         else:
            for p in process_list( almnt_list ):
               yield p
         current_name = almnt.read.name
         almnt_list = [ almnt ]
   if bundle:
      yield list( process_list( almnt_list ) )
   else:
      for p in process_list( almnt_list ):
         yield p
   if mate_missing_count[0] > 1:
      warnings.warn( "%d reads with missing mate encountered." % mate_missing_count[0] )


def pair_SAM_alignments_with_buffer( alignments, max_buffer_size=3000000 ):

   almnt_buffer = {}
   ambiguous_pairing_counter = 0
   for almnt in alignments:

      if not almnt.paired_end:
         raise ValueError, "Sequence of paired-end alignments expected, but got single-end alignment."
      if almnt.pe_which == "unknown":
         raise ValueError, "Cannot process paired-end alignment found with 'unknown' 'pe_which' status."

      matekey = ( 
         almnt.read.name, 
         "second" if almnt.pe_which == "first" else "first",
         almnt.mate_start.chrom if almnt.mate_aligned else None, 
         almnt.mate_start.pos if almnt.mate_aligned else None, 
         almnt.iv.chrom if almnt.aligned else None, 
         almnt.iv.start if almnt.aligned else None, 
         -almnt.inferred_insert_size if almnt.aligned and almnt.mate_aligned else None )

      if matekey in almnt_buffer:
         if len( almnt_buffer[ matekey ] ) == 1:            
            mate = almnt_buffer[ matekey ][ 0 ]
            del almnt_buffer[ matekey ]
         else:
            mate = almnt_buffer[ matekey ].pop( 0 )
            if ambiguous_pairing_counter == 0:
               ambiguous_pairing_first_occurance = matekey
            ambiguous_pairing_counter += 1
         if almnt.pe_which == "first":
            yield ( almnt, mate )
         else:
            yield ( mate, almnt )
      else:
         almntkey = ( 
            almnt.read.name, almnt.pe_which, 
            almnt.iv.chrom if almnt.aligned else None, 
            almnt.iv.start if almnt.aligned else None, 
            almnt.mate_start.chrom if almnt.mate_aligned else None, 
            almnt.mate_start.pos if almnt.mate_aligned else None, 
            almnt.inferred_insert_size if almnt.aligned and almnt.mate_aligned else None )
         if almntkey not in almnt_buffer:
            almnt_buffer[ almntkey ] = [ almnt ]
         else:
            almnt_buffer[ almntkey ].append( almnt )
         if len(almnt_buffer) > max_buffer_size:
            raise ValueError, "Maximum alignment buffer size exceeded while pairing SAM alignments."

   if len(almnt_buffer) > 0:
      warnings.warn( "Mate records missing for %d records; first such record: %s." % 
         ( len(almnt_buffer), str( almnt_buffer.values()[0][0] ) ) )
      for almnt_list in almnt_buffer.values():
         for almnt in almnt_list:
            if almnt.pe_which == "first":
               yield ( almnt, None )
            else:
               yield ( None, almnt )

   if ambiguous_pairing_counter > 0:
      warnings.warn( "Mate pairing was ambiguous for %d records; mate key for first such record: %s." %
         ( ambiguous_pairing_counter, str( ambiguous_pairing_first_occurance ) ) )


###########################
##   variant calls
###########################


_re_vcf_meta_comment = re.compile( "^##([a-zA-Z]+)\=(.*)$" )

_re_vcf_meta_descr = re.compile('ID=[^,]+,?|Number=[^,]+,?|Type=[^,]+,?|Description="[^"]+",?')

_re_vcf_meta_types = re.compile( "[INFO|FILTER|FORMAT]" )

_vcf_typemap = {
    "Integer":int,
    "Float":float,
    "String":str,
    "Flag":bool
}

class VariantCall( object ):
    
    def __init__( self, chrom = None, pos = None, identifier = None, ref = None, alt = None, qual = None, filtr = None, info = None ):
        self.chrom  = chrom
        self.pos    = pos
        self.id     = identifier
        self.ref    = ref
        self.alt    = alt
        self.qual   = qual
        self.filter = filtr
        self.info   = info
        self._original_line = None
    
    @classmethod
    def fromdict( cls, dictionary ):
        ret = cls()
        ret.chrom   = dictionary["chrom"]
        ret.pos     = dictionary["pos"]
        ret.id      = dictionary["id"]
        ret.ref     = dictionary["ref"]
        ret.alt     = dictionary["alt"]
        ret.qual    = dictionary["qual"]
        ret.filter  = dictionary["filter"]
        ret.info    = dictionary["info"]
        ret._original_line = None

    @classmethod
    def fromline( cls, line, nsamples = 0, sampleids = [] ):
        ret = cls()
        if nsamples == 0:
            ret.format = None
            ret.chrom, ret.pos, ret.id, ret.ref, ret.alt, ret.qual, ret.filter, ret.info = line.rstrip("\n").split("\t", 7)
        else:
            lsplit = line.rstrip("\n").split("\t")
            ret.chrom, ret.pos, ret.id, ret.ref, ret.alt, ret.qual, ret.filter, ret.info = lsplit[:8]
            ret.format = lsplit[8].split(":")
            ret.samples = {}
            spos=9
            for sid in sampleids:
                ret.samples[ sid ] = dict( ( name, value ) for (name, value) in itertools.izip( ret.format, lsplit[spos].split(":") ) )
                spos += 1
        ret.pos = GenomicPosition( ret.chrom, int(ret.pos) )
        ret.alt = ret.alt.split(",")
        ret._original_line = line
        return ret
    
    def infoline( self ):
        if self.info.__class__ == dict:
            return ";".join(map((lambda key: str(key) + "=" + str(self.info[key])), self.info ))
        else:
            return self.info
    
    def get_original_line( self ):
       warnings.warn( "Original line is empty, probably this object was created from scratch and not from a line in a .vcf file!" )
       return self._original_line
    
    def sampleline( self ):
       if self.format == None:
          print >> sys.stderr, "No samples in this variant call!" 
          return ""
       keys = self.format
       ret = [ ":".join( keys ) ]
       for sid in self.samples:
          tmp = []
          for k in keys:
             if k in self.samples[sid]:
                tmp.append( self.samples[sid][k] )
          ret.append( ":".join(tmp) )
       return "\t".join( ret )
    
    def to_line( self ):
       if self.format == None:
          return "\t".join( map( str, [ self.pos.chrom, self.pos.pos, self.id, self.ref, ",".join( self.alt ), self.qual, self.filter, self.infoline() ] ) ) + "\n"
       else:
          return "\t".join( map( str, [ self.pos.chrom, self.pos.pos, self.id, self.ref, ",".join( self.alt ), self.qual, self.filter, self.infoline(), self.sampleline() ] ) ) + "\n"
    
    def __descr__( self ):
        return "<VariantCall at %s, ref '%s', alt %s >" % (str(self.pos).rstrip("/."), self.ref, str(self.alt).strip("[]"))
    
    def __str__( self ):
        return "%s:'%s'->%s" % (str(self.pos).rstrip("/."), self.ref, str(self.alt).strip("[]"))
    
    def unpack_info( self, infodict ):
        tmp = {}
        for token in self.info.strip(";").split(";"):
            if re.compile("=").search(token):
                token = token.split("=")
                if infodict.has_key( token[0] ):
                    tmp[token[0]] = map( infodict[token[0]], token[1].split(",") )
                else:
                    tmp[token[0]] = token[1].split(",")
                if len( tmp[ token[0] ] ) == 1:
                    tmp[token[0]] = tmp[token[0]][0]
            else: #Flag attribute found
                tmp[token] = True
        diff = set( infodict.keys() ).difference( set( tmp.keys() ) )
        for key in diff:
            if infodict[key] == bool:
                tmp[key] = False
        self.info = tmp

class VCF_Reader( FileOrSequence ):

    def __init__( self, filename_or_sequence ):
        FileOrSequence.__init__( self, filename_or_sequence )
        self.metadata = {}
        self.info = {}
        self.filters = {}
        self.formats = {}
        self.nsamples = 0
        self.sampleids = []
        
    def make_info_dict( self ):
        self.infodict = dict( ( key, _vcf_typemap[self.info[key]["Type"]] ) for key in self.info.keys() )
    
    def parse_meta( self, header_filename = None ):
        if header_filename == None:
            the_iter = FileOrSequence.__iter__( self )
        else:
            the_iter = open( header_filename, "r" )
        
        for line in the_iter:
            if line.startswith( '#' ):
                if line.startswith( "##" ):
                    mo = _re_vcf_meta_comment.match( line )
                    if mo:
                        value = mo.group(2)
                        if mo.group(1) == "INFO":
                            value = dict( e.rstrip(",").split("=",1) for e in _re_vcf_meta_descr.findall(value) )
                            key = value["ID"]
                            del value["ID"]
                            self.info[ key ] = value
                        elif mo.group(1) == "FILTER":
                            value = dict( e.rstrip(",").split("=",1) for e in _re_vcf_meta_descr.findall(value) )
                            key = value["ID"]
                            del value["ID"]
                            self.filters[ key ] = value
                        elif mo.group(1) == "FORMAT":
                            value = dict( e.rstrip(",").split("=",1) for e in _re_vcf_meta_descr.findall(value) )
                            key = value["ID"]
                            del value["ID"]
                            self.formats[ key ] = value
                        else:
                            self.metadata[ mo.group(1) ] = mo.group(2)
                else:
                    self.sampleids = line.rstrip("\t\n").split("\t")[9:]
                    self.nsamples = len( self.sampleids )
                continue
            else:
                break
    
    def meta_info( self, header_filename = None ):
       ret = []
       if header_filename == None:
          the_iter = FileOrSequence.__iter__( self )
       else:
          the_iter = open( header_filename, "r" )
       
       for line in the_iter:
          if line.startswith( '#' ):
             ret.append( line )
          else:
             break
       return ret
    
    def __iter__( self ):
        for line in FileOrSequence.__iter__( self ):
            if line == "\n" or line.startswith( '#' ):
                continue
            vc = VariantCall.fromline( line, self.nsamples, self.sampleids )
            yield vc


class WiggleReader( FileOrSequence ):

    def __init__( self, filename_or_sequence, verbose = True ):
        FileOrSequence.__init__( self, filename_or_sequence )
        self.attributes = {}       
        self.stepType = 'none'
        self.verbose = verbose
        
    def __iter__( self ):
        span = 1
        pos = None
        step = None
        chrom = None
        for line in FileOrSequence.__iter__( self ):
            if line.startswith( 'track' ):
                fields = shlex.split(line)[1:]
                self.attributes = dict([(p[0], p[1].strip('"')) for p in [x.split("=") for x in fields]])
            elif line.startswith( 'fixedStep' ): # do fixed step stuff
                self.stepType = 'fixed'
                fields = shlex.split(line)[1:]
                declarations = dict([(p[0], p[1].strip('"')) for p in [x.split("=") for x in fields]])
                pos = int(declarations['start'])
                step = int(declarations['step'])
                chrom = declarations['chrom']
                if 'span' in declarations:
                    span = int(declarations['span'])
                else:
                    span = 1
            elif line.startswith( 'variableStep' ): # do variable step stuff
                self.stepType = 'variable'
                fields = shlex.split(line)[1:]
                declarations = dict([(p[0], p[1].strip('"')) for p in [x.split("=") for x in fields]])
                chrom = declarations['chrom']
                if 'span' in declarations:
                    span = int(declarations['span'])
                else:
                    span = 1
            elif line.startswith( 'browser' ) or line.startswith( '#' ): #Comment or ignored
                if self.verbose:
                    print "Ignored line:", line
                continue
            else:
                if self.stepType == 'fixed':
                    yield ( GenomicInterval( chrom, pos, pos + span, '.' ), float(line.strip()) )
                    pos += step
                elif self.stepType == 'variable':
                    tmp = line.strip().split(" ")
                    pos = int(tmp[0])
                    yield ( GenomicInterval( chrom, pos, pos + span, '.' ), float(tmp[1]) )
            
class BAM_Reader( object ):

    def __init__( self, filename ):
        global pysam
        self.filename = filename
        self.sf = None  # This one is only used by __getitem__
        self.record_no = -1
        try:
           import pysam
        except ImportError:
           sys.stderr.write( "Please Install PySam to use the BAM_Reader Class (http://code.google.com/p/pysam/)" )
           raise
    
    def __iter__( self ):
        sf = pysam.Samfile(self.filename, "rb")
        self.record_no = 0
        for pa in sf:
            yield SAM_Alignment.from_pysam_AlignedRead( pa, sf )
            self.record_no += 1
    
    def fetch( self, reference = None, start = None, end = None, region = None ):
        sf = pysam.Samfile(self.filename, "rb")
        self.record_no = 0
        try:
           for pa in sf.fetch( reference, start, end, region ):
              yield SAM_Alignment.from_pysam_AlignedRead( pa, sf )
              self.record_no += 1
        except ValueError as e:
           if e.message == "fetch called on bamfile without index":
              print "Error: ", e.message
              print "Your bam index file is missing or wrongly named, convention is that file 'x.bam' has index file 'x.bam.bai'!"
           else:
              raise
        except:
           raise

    def get_line_number_string( self ):
        if self.record_no == -1:
            return "unopened file %s" % ( self.filename )
        else:
            return "record #%d in file %s" % ( self.record_no, self.filename )
    
    def __getitem__( self, iv ):
        if not isinstance( iv, GenomicInterval ):
           raise TypeError, "Use a HTSeq.GenomicInterval to access regions within .bam-file!"        
        if self.sf is None:
           self.sf = pysam.Samfile( self.filename, "rb" )
           if not self.sf._hasIndex():
              raise ValueError, "The .bam-file has no index, random-access is disabled!"
        for pa in self.sf.fetch( iv.chrom, iv.start+1, iv.end ):
            yield SAM_Alignment.from_pysam_AlignedRead( pa, self.sf )
    
    def get_header_dict( self ):
       sf = pysam.Samfile(self.filename, "rb")
       return sf.header
    
               
class BAM_Writer( object ):
   def __init__( self, filename, template = None, referencenames = None, referencelengths = None, text = None, header = None ):
      try:
         import pysam
      except ImportError:
         sys.stderr.write( "Please Install PySam to use the BAM_Writer Class (http://code.google.com/p/pysam/)" )
         raise
      
      self.filename = filename
      self.template = template
      self.referencenames = referencenames
      self.referencelengths = referencelengths
      self.text = text
      self.header = header
      self.sf = pysam.Samfile( self.filename, mode="wb", template = self.template, referencenames = self.referencenames, referencelengths = self.referencelengths, text = self.text, header = self.header )
      
   @classmethod
   def from_BAM_Reader( cls, fn, br ):
      return BAM_Writer( filename = fn, header = br.get_header_dict() )
   
   def write( self, alnmt):
      self.sf.write( alnmt.to_pysam_AlignedRead( self.sf ) )
   
   def close( self ):
      self.sf.close()


class BED_Reader( FileOrSequence ):

   def __init__( self, filename_or_sequence ):
      FileOrSequence.__init__( self, filename_or_sequence )
        
   def __iter__( self ):
      for line in FileOrSequence.__iter__( self ):
         if line.startswith( "track" ):
            continue
         fields = line.split()
         if len(fields) < 3:
            raise ValueError, "BED file line contains less than 3 fields"
         if len(fields) > 9:
            raise ValueError, "BED file line contains more than 9 fields"
         iv = GenomicInterval( fields[0], int(fields[1]), int(fields[2]), fields[5] if len(fields) > 5 else "." )
         f = GenomicFeature( fields[3] if len(fields) > 3 else "unnamed", "BED line", iv )
         f.score = float( fields[4] ) if len(fields) > 4 else None
         f.thick = GenomicInterval( iv.chrom, int( fields[6] ), int( fields[7] ), iv.strand ) if len(fields) > 7 else None
         f.itemRgb = [ int(a) for a in fields[8].split(",") ]  if len(fields) > 8 else None
         yield(f)




#!/usr/bin/env python

# HTSeq_QA.py
#
# (c) Simon Anders, European Molecular Biology Laboratory, 2010
# released under GNU General Public License

import sys, time, os.path, optparse
from itertools import *
import numpy
import HTSeq

def main():

   try:
      import matplotlib
   except ImportError:
      sys.stderr.write("This script needs the 'matplotlib' library, which ")
      sys.stderr.write("was not found. Please install it." )
   matplotlib.use('PDF')
   from matplotlib import pyplot


   # **** Parse command line ****

   optParser = optparse.OptionParser( usage = "%prog [options] read_file",
      description=
	 "This script take a file with high-throughput sequencing reads " +
	 "(supported formats: SAM, Solexa _export.txt, FASTQ, Solexa " +
	 "_sequence.txt) and performs a simply quality assessment by " +
	 "producing plots showing the distribution of called bases and " +
	 "base-call quality scores by position within the reads. The " +
	 "plots are output as a PDF file.",
      epilog = 
	 "Written by Simon Anders (sanders@fs.tum.de), European Molecular Biology " +
	 " Laboratory (EMBL). (c) 2010. Released under the terms of the GNU General " +
	 " Public License v3. Part of the 'HTSeq' framework, version %s." % HTSeq.__version__ )
   optParser.add_option( "-t", "--type", type="choice", dest="type",
      choices = ("sam", "bam", "solexa-export", "fastq", "solexa-fastq"),
      default = "sam", help="type of read_file (one of: sam [default], bam, " +
	 "solexa-export, fastq, solexa-fastq)" )
   optParser.add_option( "-o", "--outfile", type="string", dest="outfile",
      help="output filename (default is <read_file>.pdf)" )
   optParser.add_option( "-r", "--readlength", type="int", dest="readlen",
      help="the maximum read length (when not specified, the script guesses from the file" )
   optParser.add_option( "-g", "--gamma", type="float", dest="gamma", 
      default = 0.3,
      help="the gamma factor for the contrast adjustment of the quality score plot" )
   optParser.add_option( "-n", "--nosplit", action="store_true", dest="nosplit",
      help="do not split reads in unaligned and aligned ones" )
   optParser.add_option( "-m", "--maxqual", type="int", dest="maxqual", default=41,
      help="the maximum quality score that appears in the data (default: 41)" )

   if len( sys.argv ) == 1:
      optParser.print_help()
      sys.exit(1)

   (opts, args) = optParser.parse_args()

   if len( args ) != 1:
      sys.stderr.write( sys.argv[0] + ": Error: Please provide one argument (the read_file).\n" )
      sys.stderr.write( "  Call with '-h' to get usage information.\n" )
      sys.exit( 1 )

   readfilename = args[0]

   if opts.type == "sam":
      readfile = HTSeq.SAM_Reader( readfilename )
      isAlnmntFile = True
   elif opts.type == "bam":
      readfile = HTSeq.BAM_Reader( readfilename )
      isAlnmntFile = True
   elif opts.type == "solexa-export":
      readfile = HTSeq.SolexaExportReader( readfilename )
      isAlnmntFile = True
   elif opts.type == "fastq":
      readfile = HTSeq.FastqReader( readfilename )
      isAlnmntFile = False
   elif opts.type == "solexa-fastq":
      readfile = HTSeq.FastqReader( readfilename, "solexa" )
      isAlnmntFile = False
   else:
      sys.error( "Oops." )

   twoColumns = isAlnmntFile and not opts.nosplit

   if opts.outfile is None:
      outfilename = os.path.basename( readfilename ) + ".pdf"
   else:   
      outfilename = opts.outfile


   # **** Get read length ****

   if opts.readlen is not None:
      readlen = opts.readlen
   else:
      readlen = 0
      if isAlnmntFile:
	 reads = ( a.read for a in readfile )
      else:
	 reads = readfile
      for r in islice( reads, 10000 ):
	 if len( r ) > readlen:
            readlen = len( r )

   max_qual = opts.maxqual
   gamma = opts.gamma


   # **** Initialize count arrays ****

   base_arr_U = numpy.zeros( ( readlen, 5 ), numpy.int )
   qual_arr_U = numpy.zeros( ( readlen, max_qual+1 ), numpy.int )
   if twoColumns:
      base_arr_A = numpy.zeros( ( readlen, 5 ), numpy.int )
      qual_arr_A = numpy.zeros( ( readlen, max_qual+1 ), numpy.int )


   # **** Main counting loop ****

   i = 0
   try:
      for a in readfile:
	 if isAlnmntFile:
	    r = a.read
	 else:
	    r = a
	 if twoColumns and (isAlnmntFile and a.aligned):
	    r.add_bases_to_count_array( base_arr_A )
	    r.add_qual_to_count_array( qual_arr_A )
	 else:
	    r.add_bases_to_count_array( base_arr_U )
	    r.add_qual_to_count_array( qual_arr_U )   
	 i += 1
	 if i % 200000 == 0:
            print i, "reads processed"
   except:
      sys.stderr.write( "Error occured in: %s\n" %
         readfile.get_line_number_string() )
      raise
   print i, "reads processed"


   # **** Normalize result ****

   def norm_by_pos( arr ):
      arr = numpy.array( arr, numpy.float )
      arr_n = ( arr.T / arr.sum( 1 ) ).T
      arr_n[ arr == 0 ] = 0
      return arr_n

   def norm_by_start( arr ):
      arr = numpy.array( arr, numpy.float )
      arr_n = ( arr.T / arr.sum( 1 )[ 0 ] ).T
      arr_n[ arr == 0 ] = 0
      return arr_n


   base_arr_U_n = norm_by_pos( base_arr_U )
   qual_arr_U_n = norm_by_start( qual_arr_U )
   nreads_U = base_arr_U[0,:].sum()
   if twoColumns:
      base_arr_A_n = norm_by_pos( base_arr_A )
      qual_arr_A_n = norm_by_start( qual_arr_A )
      nreads_A = base_arr_A[0,:].sum()


   # **** Make plot ****

   def plot_bases( arr ):
      xg = numpy.arange( readlen )   
      pyplot.plot( xg, arr[ : , 0 ], marker='.', color='red')
      pyplot.plot( xg, arr[ : , 1 ], marker='.', color='darkgreen')
      pyplot.plot( xg, arr[ : , 2 ], marker='.',color='lightgreen')
      pyplot.plot( xg, arr[ : , 3 ], marker='.',color='orange')
      pyplot.plot( xg, arr[ : , 4 ], marker='.',color='grey')
      pyplot.axis( (0, readlen-1, 0, 1 ) )
      pyplot.text( readlen*.70, .9, "A", color="red" )
      pyplot.text( readlen*.75, .9, "C", color="darkgreen" )
      pyplot.text( readlen*.80, .9, "G", color="lightgreen" )
      pyplot.text( readlen*.85, .9, "T", color="orange" )
      pyplot.text( readlen*.90, .9, "N", color="grey" )

   pyplot.figure()
   pyplot.subplots_adjust( top=.85 )
   pyplot.suptitle( os.path.basename(readfilename), fontweight='bold' )

   if twoColumns:

      pyplot.subplot( 221 )
      plot_bases( base_arr_U_n )
      pyplot.ylabel( "proportion of base" )
      pyplot.title( "non-aligned reads\n%.0f%% (%.3f million)" % 
	 ( 100. * nreads_U / (nreads_U+nreads_A), nreads_U / 1e6 ) )

      pyplot.subplot( 222 )
      plot_bases( base_arr_A_n )
      pyplot.title( "aligned reads\n%.0f%% (%.3f million)" % 
	 ( 100. * nreads_A / (nreads_U+nreads_A), nreads_A / 1e6 ) )

      pyplot.subplot( 223 )
      pyplot.pcolor( qual_arr_U_n.T ** gamma, cmap=pyplot.cm.Greens,
	  norm=pyplot.normalize( 0, 1 ) )
      pyplot.axis( (0, readlen-1, 0, max_qual+1 ) )
      pyplot.xlabel( "position in read" )
      pyplot.ylabel( "base-call quality score" )

      pyplot.subplot( 224 )
      pyplot.pcolor( qual_arr_A_n.T ** gamma, cmap=pyplot.cm.Greens,
	   norm=pyplot.normalize( 0, 1 ) )
      pyplot.axis( (0, readlen-1, 0, max_qual+1 ) )
      pyplot.xlabel( "position in read" )

   else:

      pyplot.subplot( 211 )
      plot_bases( base_arr_U_n )
      pyplot.ylabel( "proportion of base" )
      pyplot.title( "%.3f million reads" % ( nreads_U / 1e6 ) )

      pyplot.subplot( 212 )
      pyplot.pcolor( qual_arr_U_n.T ** gamma, cmap=pyplot.cm.Greens,
	  norm=pyplot.normalize( 0, 1 ) )
      pyplot.axis( (0, readlen-1, 0, max_qual+1 ) )
      pyplot.xlabel( "position in read" )
      pyplot.ylabel( "base-call quality score" )


   pyplot.savefig( outfilename )

if __name__ == "__main__":
   main()



import HTSeq
import numpy
from matplotlib import pyplot

sortedbamfile = HTSeq.BAM_Reader( "SRR001432_head_sorted.bam" )
gtffile = HTSeq.GFF_Reader( "Homo_sapiens.GRCh37.56_chrom1.gtf" )
halfwinwidth = 3000
fragmentsize = 200

tsspos = set()
for feature in gtffile:
   if feature.type == "exon" and feature.attr["exon_number"] == "1":
      tsspos.add( feature.iv.start_d_as_pos )

profile = numpy.zeros( 2*halfwinwidth, dtype='i' )   
for p in tsspos:
   window = HTSeq.GenomicInterval( p.chrom, 
       p.pos - halfwinwidth - fragmentsize, p.pos + halfwinwidth + fragmentsize, "." )
   for almnt in sortedbamfile[ window ]:
      almnt.iv.length = fragmentsize
      if p.strand == "+":
         start_in_window = almnt.iv.start - p.pos + halfwinwidth 
         end_in_window   = almnt.iv.end   - p.pos + halfwinwidth 
      else:
         start_in_window = p.pos + halfwinwidth - almnt.iv.end
         end_in_window   = p.pos + halfwinwidth - almnt.iv.start
      start_in_window = max( start_in_window, 0 )
      end_in_window = min( end_in_window, 2*halfwinwidth )
      if start_in_window >= 2*halfwinwidth or end_in_window < 0:
         continue
      profile[ start_in_window : end_in_window ] += 1




import sys, optparse, itertools, warnings, traceback, os.path

import HTSeq

class UnknownChrom( Exception ):
   pass
   
def invert_strand( iv ):
   iv2 = iv.copy()
   if iv2.strand == "+":
      iv2.strand = "-"
   elif iv2.strand == "-":
      iv2.strand = "+"
   else:
      raise ValueError, "Illegal strand"
   return iv2

def count_reads_in_features( sam_filename, gff_filename, samtype, order, stranded, 
      overlap_mode, feature_type, id_attribute, quiet, minaqual, samout ):
      
   def write_to_samout( r, assignment ):
      if samoutfile is None:
         return
      if not pe_mode:
         r = (r,)
      for read in r:
         if read is not None:
            samoutfile.write( read.original_sam_line.rstrip() + 
               "\tXF:Z:" + assignment + "\n" )
      

   if samout != "":
      samoutfile = open( samout, "w" )
   else:
      samoutfile = None
      
   features = HTSeq.GenomicArrayOfSets( "auto", stranded != "no" )     
   counts = {}

   # Try to open samfile to fail early in case it is not there
   if sam_filename != "-":
      open( sam_filename ).close()
      
   gff = HTSeq.GFF_Reader( gff_filename )   
   i = 0
   try:
      for f in gff:
         if f.type == feature_type:
            try:
               feature_id = f.attr[ id_attribute ]
            except KeyError:
               raise ValueError, ( "Feature %s does not contain a '%s' attribute" % 
                  ( f.name, id_attribute ) )
            if stranded != "no" and f.iv.strand == ".":
               raise ValueError, ( "Feature %s at %s does not have strand information but you are "
                  "running htseq-count in stranded mode. Use '--stranded=no'." % 
                  ( f.name, f.iv ) )
            features[ f.iv ] += feature_id
            counts[ f.attr[ id_attribute ] ] = 0
         i += 1
         if i % 100000 == 0 and not quiet:
            sys.stderr.write( "%d GFF lines processed.\n" % i )
   except:
      sys.stderr.write( "Error occured when processing GFF file (%s):\n" % gff.get_line_number_string() )
      raise
      
   if not quiet:
      sys.stderr.write( "%d GFF lines processed.\n" % i )
      
   if len( counts ) == 0:
      sys.stderr.write( "Warning: No features of type '%s' found.\n" % feature_type )
   
   if samtype == "sam":
      SAM_or_BAM_Reader = HTSeq.SAM_Reader
   elif samtype == "bam":
      SAM_or_BAM_Reader = HTSeq.BAM_Reader
   else:
      raise ValueError, "Unknown input format %s specified." % samtype

   try:
      if sam_filename != "-":
         read_seq_file = SAM_or_BAM_Reader( sam_filename )
         read_seq = read_seq_file
         first_read = iter(read_seq).next()
      else:
         read_seq_file = SAM_or_BAM_Reader( sys.stdin )
         read_seq_iter = iter( read_seq_file )
         first_read = read_seq_iter.next()
         read_seq = itertools.chain( [ first_read ], read_seq_iter )
      pe_mode = first_read.paired_end
   except:
      sys.stderr.write( "Error occured when reading beginning of SAM/BAM file.\n" )
      raise

   try:
      if pe_mode:
         if order == "name":
            read_seq = HTSeq.pair_SAM_alignments( read_seq )
         elif order == "pos":
            read_seq = HTSeq.pair_SAM_alignments_with_buffer( read_seq )
         else:
            raise ValueError, "Illegal order specified."
      empty = 0
      ambiguous = 0
      notaligned = 0
      lowqual = 0
      nonunique = 0
      i = 0   
      for r in read_seq:
         if i > 0 and i % 100000 == 0 and not quiet:
            sys.stderr.write( "%d SAM alignment record%s processed.\n" % ( i, "s" if not pe_mode else " pairs" ) )

         i += 1
         if not pe_mode:
            if not r.aligned:
               notaligned += 1
               write_to_samout( r, "__not_aligned" )
               continue
            try:
               if r.optional_field( "NH" ) > 1:
                  nonunique += 1
                  write_to_samout( r, "__alignment_not_unique" )
                  continue
            except KeyError:
               pass
            if r.aQual < minaqual:
               lowqual += 1
               write_to_samout( r, "__too_low_aQual" )
               continue
            if stranded != "reverse":
               iv_seq = ( co.ref_iv for co in r.cigar if co.type == "M" and co.size > 0 )
            else:
               iv_seq = ( invert_strand( co.ref_iv ) for co in r.cigar if co.type == "M" and co.size > 0 )            
         else:
            if r[0] is not None and r[0].aligned:
               if stranded != "reverse":
                  iv_seq = ( co.ref_iv for co in r[0].cigar if co.type == "M" and co.size > 0 )
               else:
                  iv_seq = ( invert_strand( co.ref_iv ) for co in r[0].cigar if co.type == "M" and co.size > 0 )
            else:
               iv_seq = tuple()
            if r[1] is not None and r[1].aligned:            
               if stranded != "reverse":
                  iv_seq = itertools.chain( iv_seq, 
                     ( invert_strand( co.ref_iv ) for co in r[1].cigar if co.type == "M" and co.size > 0 ) )
               else:
                  iv_seq = itertools.chain( iv_seq, 
                     ( co.ref_iv for co in r[1].cigar if co.type == "M" and co.size > 0 ) )
            else:
               if ( r[0] is None ) or not ( r[0].aligned ):
                  write_to_samout( r, "__not_aligned" )
                  notaligned += 1
                  continue         
            try:
               if ( r[0] is not None and r[0].optional_field( "NH" ) > 1 ) or \
                     ( r[1] is not None and r[1].optional_field( "NH" ) > 1 ):
                  nonunique += 1
                  write_to_samout( r, "__alignment_not_unique" )
                  continue
            except KeyError:
               pass
            if ( r[0] and r[0].aQual < minaqual ) or ( r[1] and r[1].aQual < minaqual ):
               lowqual += 1
               write_to_samout( r, "__too_low_aQual" )
               continue         
         
         try:
            if overlap_mode == "union":
               fs = set()
               for iv in iv_seq:
                  if iv.chrom not in features.chrom_vectors:
                     raise UnknownChrom
                  for iv2, fs2 in features[ iv ].steps():
                     fs = fs.union( fs2 )
            elif overlap_mode == "intersection-strict" or overlap_mode == "intersection-nonempty":
               fs = None
               for iv in iv_seq:
                  if iv.chrom not in features.chrom_vectors:
                     raise UnknownChrom
                  for iv2, fs2 in features[ iv ].steps():
                     if len(fs2) > 0 or overlap_mode == "intersection-strict":
                        if fs is None:
                           fs = fs2.copy()
                        else:
                           fs = fs.intersection( fs2 )
            else:
               sys.exit( "Illegal overlap mode." )
            if fs is None or len( fs ) == 0:
               write_to_samout( r, "__no_feature" )
               empty += 1
            elif len( fs ) > 1:
               write_to_samout( r, "__ambiguous[" + '+'.join( fs ) + "]" )
               ambiguous += 1
            else:
               write_to_samout( r, list(fs)[0] )
               counts[ list(fs)[0] ] += 1
         except UnknownChrom:
            write_to_samout( r, "__no_feature" )
            empty += 1

   except:
      sys.stderr.write( "Error occured when processing SAM input (%s):\n" % read_seq_file.get_line_number_string() )
      raise

   if not quiet:
      sys.stderr.write( "%d SAM %s processed.\n" % ( i, "alignments " if not pe_mode else "alignment pairs" ) )
         
   if samoutfile is not None:
      samoutfile.close()

   for fn in sorted( counts.keys() ):
      print "%s\t%d" % ( fn, counts[fn] )
   print "__no_feature\t%d" % empty
   print "__ambiguous\t%d" % ambiguous
   print "__too_low_aQual\t%d" % lowqual
   print "__not_aligned\t%d" % notaligned
   print "__alignment_not_unique\t%d" % nonunique

      
def main():
   
   optParser = optparse.OptionParser( 
      
      usage = "%prog [options] alignment_file gff_file",
      
      description=
         "This script takes an alignment file in SAM/BAM format and a " +
         "feature file in GFF format and calculates for each feature " +
         "the number of reads mapping to it. See " +
         "http://www-huber.embl.de/users/anders/HTSeq/doc/count.html for details." ,
         
      epilog = 
         "Written by Simon Anders (sanders@fs.tum.de), European Molecular Biology " +
         "Laboratory (EMBL). (c) 2010. Released under the terms of the GNU General " +
         "Public License v3. Part of the 'HTSeq' framework, version %s." % HTSeq.__version__ )
         
   optParser.add_option( "-f", "--format", type="choice", dest="samtype",
      choices = ( "sam", "bam" ), default = "sam",
      help = "type of <alignment_file> data, either 'sam' or 'bam' (default: sam)" )

   optParser.add_option( "-r", "--order", type="choice", dest="order",
     choices=("pos", "name"), default="name",
     help = "'pos' or 'name'. Sorting order of <alignment_file> (default: name). Paired-end sequencing " +
        "data must be sorted either by position or by read name, and the sorting order " +
        "must be specified. Ignored for single-end data." )

   optParser.add_option( "-s", "--stranded", type="choice", dest="stranded",
     choices = ( "yes", "no", "reverse" ), default = "yes",
     help = "whether the data is from a strand-specific assay. Specify 'yes', " +
        "'no', or 'reverse' (default: yes). " +
        "'reverse' means 'yes' with reversed strand interpretation" )
      
   optParser.add_option( "-a", "--minaqual", type="int", dest="minaqual",
      default = 10,
      help = "skip all reads with alignment quality lower than the given " +
         "minimum value (default: 10)" )
      
   optParser.add_option( "-t", "--type", type="string", dest="featuretype",
      default = "exon", help = "feature type (3rd column in GFF file) to be used, " +
         "all features of other type are ignored (default, suitable for Ensembl " +
         "GTF files: exon)" )
         
   optParser.add_option( "-i", "--idattr", type="string", dest="idattr",
      default = "gene_id", help = "GFF attribute to be used as feature ID (default, " +
      "suitable for Ensembl GTF files: gene_id)" )

   optParser.add_option( "-m", "--mode", type="choice", dest="mode",
      choices = ( "union", "intersection-strict", "intersection-nonempty" ), 
      default = "union", help = "mode to handle reads overlapping more than one feature " +
         "(choices: union, intersection-strict, intersection-nonempty; default: union)" )
         
   optParser.add_option( "-o", "--samout", type="string", dest="samout",
      default = "", help = "write out all SAM alignment records into an output " +
      "SAM file called SAMOUT, annotating each line with its feature assignment " +
      "(as an optional field with tag 'XF')" )

   optParser.add_option( "-q", "--quiet", action="store_true", dest="quiet",
      help = "suppress progress report" ) # and warnings" )

   if len( sys.argv ) == 1:
      optParser.print_help()
      sys.exit(1)

   (opts, args) = optParser.parse_args()

   if len( args ) != 2:
      sys.stderr.write( sys.argv[0] + ": Error: Please provide two arguments.\n" )
      sys.stderr.write( "  Call with '-h' to get usage information.\n" )
      sys.exit( 1 )
      
   warnings.showwarning = my_showwarning
   try:
      count_reads_in_features( args[0], args[1], opts.samtype, opts.order, opts.stranded, 
         opts.mode, opts.featuretype, opts.idattr, opts.quiet, opts.minaqual,
         opts.samout )
   except:
      sys.stderr.write( "  %s\n" % str( sys.exc_info()[1] ) )
      sys.stderr.write( "  [Exception type: %s, raised in %s:%d]\n" % 
         ( sys.exc_info()[1].__class__.__name__, 
           os.path.basename(traceback.extract_tb( sys.exc_info()[2] )[-1][0]), 
           traceback.extract_tb( sys.exc_info()[2] )[-1][1] ) )
      sys.exit( 1 )

def my_showwarning( message, category, filename, lineno = None, line = None ):
   sys.stderr.write( "Warning: %s\n" % message )

if __name__ == "__main__":
   main()




import sys, os
import distutils.util
import numpy

build_dir = "build/lib.%s-%s" % ( distutils.util.get_platform(), sys.version[0:3] )

sys.path.insert( 0, os.path.join( os.getcwd(), build_dir ) )
import HTSeq
os.chdir( "example_data" )

print "TSS test, scheme 1"
execfile( os.path.join( "..", "doc", "tss1.py" ) )
profile1 = profile.copy()
print "finished"

print "TSS test, scheme 2"
execfile( os.path.join( "..", "doc", "tss2.py" ) )
profile2 = profile.copy()
print "finished",

if ( profile2 == profile1 ).all():
   print "and matches result from scheme 1."
else:   
   print "and differs to result from scheme 1!  <<<!!!>>>"

print "TSS test, scheme 3"
execfile( os.path.join( "..", "doc", "tss3.py" ) )
profile3 = profile.copy()
print "finished",

if ( profile3 == profile1 ).all():
   print "and matches result from scheme 1."
else:   
   print "and differs to result from scheme 1!  <<<!!!>>>"




import glob

counter_code = '''
<!-- Start of StatCounter Code -->
<script type="text/javascript">
var sc_project=5680199; 
var sc_invisible=1; 
var sc_partition=63; 
var sc_click_stat=1; 
var sc_security="6e579550"; 
</script>

<script type="text/javascript"
src="http://www.statcounter.com/counter/counter.js"></script><noscript><div
class="statcounter"><a title="counter for iweb"
href="http://www.statcounter.com/iweb/" target="_blank"><img
class="statcounter"
src="http://c.statcounter.com/5680199/0/6e579550/1/"
alt="counter for iweb" ></a></div></noscript>
<!-- End of StatCounter Code -->
'''

def process_file( fn ):
   lines = open( fn ).readlines()
   for i in xrange(len(lines)):
      if lines[i].count( 'statcounter' ) > 0:
         return
   for i in xrange(len(lines)):
      if lines[i].count( '<\body>' ) > 0:
         break
   lines.insert( i, counter_code )
   f = open( fn, 'w' )
   for l in lines:
      f.write( l )
   f.close()
      
for fn in glob.glob( "_build/html/*html" ):
   process_file( fn )      

   



#!/usr/bin/env python

import sys
import os.path

try:
   from setuptools import setup, Extension
except ImportError:   
   sys.stderr.write( "Could not import 'setuptools', falling back to 'distutils'.\n" )
   from distutils.core import setup, Extension

if sys.version_info[0] < 2 or sys.version_info < 5:
   sys.stderr.write( "Error in setup script for HTSeq:\n" )
   sys.stderr.write( "You need at least version 2.5 of Python to use HTSeq.\n" )
   sys.exit( 1 )

if sys.version_info[0] >= 3:
   sys.stderr.write( "Error in setup script for HTSeq:\n" )
   sys.stderr.write( "Sorry, this package does not yet work with Python 3.\n" )
   sys.stderr.write( "Please use Python 2.x, x>=5.\n" )
   sys.exit( 1 )

try:
   import numpy
except ImportError:
   sys.stderr.write( "Setup script for HTSeq: Failed to import 'numpy'.\n" )
   sys.stderr.write( "Please install numpy and then try again to install HTSeq.\n" )
   sys.exit( 1 )
   
numpy_include_dir = os.path.join( os.path.dirname( numpy.__file__ ), 'core', 'include' )
 

setup( name = 'HTSeq',
       version = file("VERSION").readline().rstrip(),
       author = 'Simon Anders',
       author_email = 'sanders@fs.tum.de',
       url = 'http://www-huber.embl.de/users/anders/HTSeq/',
       description = "A framework to process and analyze data from " +
          "high-throughput sequencing (HTS) assays",
       classifiers = [
          'Development Status :: 5 - Production/Stable',
          'Topic :: Scientific/Engineering :: Bio-Informatics',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: GNU General Public License (GPL)',
          'Operating System :: POSIX',
          'Programming Language :: Python'
       ],
       requires = [ 'numpy', 'python (>=2.5, <3.0)' ],
       
       py_modules = [ 
          'HTSeq._HTSeq_internal', 
          'HTSeq.StepVector',
          'HTSeq._version',
          'HTSeq.scripts.qa',
          'HTSeq.scripts.count'
       ],
       ext_modules = [ 
          Extension( 'HTSeq._HTSeq', 
             ['src/_HTSeq.c'], include_dirs=[numpy_include_dir], extra_compile_args=['-w'] ),
          Extension( 'HTSeq._StepVector', 
             ['src/StepVector_wrap.cxx'], extra_compile_args=['-w'] ),
       ],
       scripts = [
          'scripts/htseq-qa',
          'scripts/htseq-count',
       ]
     )





import HTSeq
import numpy
from matplotlib import pyplot

bamfile = HTSeq.BAM_Reader( "SRR001432_head.bam" )
gtffile = HTSeq.GFF_Reader( "Homo_sapiens.GRCh37.56_chrom1.gtf" )
halfwinwidth = 3000
fragmentsize = 200

tsspos = HTSeq.GenomicArrayOfSets( "auto", stranded=False )
for feature in gtffile:
   if feature.type == "exon" and feature.attr["exon_number"] == "1":
      p = feature.iv.start_d_as_pos
      window = HTSeq.GenomicInterval( p.chrom, p.pos - halfwinwidth, p.pos + halfwinwidth, "." )
      tsspos[ window ] += p

profile = numpy.zeros( 2*halfwinwidth, dtype="i" )
for almnt in bamfile:
   if almnt.aligned:
      almnt.iv.length = fragmentsize
      s = set()
      for step_iv, step_set in tsspos[ almnt.iv ].steps():
         s |= step_set
      for p in s:
         if p.strand == "+":
            start_in_window = almnt.iv.start - p.pos + halfwinwidth
            end_in_window   = almnt.iv.end   - p.pos + halfwinwidth
         else:
            start_in_window = p.pos + halfwinwidth - almnt.iv.end
            end_in_window   = p.pos + halfwinwidth - almnt.iv.start
         start_in_window = max( start_in_window, 0 )
         end_in_window = min( end_in_window, 2*halfwinwidth )
         profile[ start_in_window : end_in_window ] += 1




import HTSeq
import itertools
import numpy

def GenomicInterval_xrange( gi, step ):
   for pos in xrange( gi.start, gi.end, step ):
      yield HTSeq.GenomicPosition( gi.chrom, pos, gi.strand )
      
def GenomicInterval_xranged( gi, step ):
   if gi.strand == "-":
      step *= -1
   for pos in xrange( gi.start_d, gi.end_d, step ):
      yield HTSeq.GenomicPosition( gi.chrom, pos, gi.strand ) 
                   
def ChromVector_steps( cv ):
   if isinstance( cv.array, numpy.ndarray ):
      start = cv.iv.start
      prev_val = None
      for i in xrange( cv.iv.start, cv.iv.end ):
         val = cv.array[ i - cv.offset ]
         if prev_val is None or val != prev_val:
            if prev_val is not None:
               yield ( HTSeq.GenomicInterval( cv.iv.chrom, start, i, cv.iv.strand ), prev_val )
            prev_val = val
            start = i
      yield ( HTSeq.GenomicInterval( cv.iv.chrom, start, cv.iv.end, cv.iv.strand ), prev_val )
   elif isinstance( cv.array, HTSeq.StepVector.StepVector ):
      for start, stop, value in cv.array[cv.iv.start:cv.iv.end].get_steps():
         yield ( HTSeq.GenomicInterval( cv.iv.chrom, start, stop, cv.iv.strand ), value )
   else:
      raise SystemError, "Unknown array type."
      
def GenomicArray_steps( ga ):
   for a in ga.chrom_vectors.values():
      for cv in a.values():
         for iv, val in cv.steps():
            yield iv, val

